@@ -226,7 +226,7 @@ class AzureML(BackendBase):
                 console.print("\nruns for experiment {}:".format(exper_name))
 
                 columns = ["xt_run_name", "status", "id", "number", "PORTAL_URL"]
-                lb = ReportBuilder(self.config, self.store, client=None)
+                lb = ReportBuilder(self.config, self.store)
 
                 if max_items and len(runs) > max_items:
                     runs = runs[:max_items]
