@@ -481,6 +481,11 @@ class BackendBase(implements(BackendInterface)):
         self.append_export(cmds, "XT_RUN_NAME", self.run_name)
         self.append_export(cmds, "XT_MOUNTING_ENABLED", self.mounting_enabled)
 
+        # for debugging version 1 vs. 2 issues
+        self.append_export(cmds, "XT_STORE_NAME", self.config.get("store"))
+        self.append_export(cmds, "XT_JOBS_CONTAINER_NAME", store_utils.get_jobs_container(args["workspace"]))
+        self.append_export(cmds, "XT_STORAGE_FORMAT", store_utils.STORAGE_FORMAT)
+        
         self.add_info_cmds(cmds, args)
 
         # debug
