@@ -237,7 +237,7 @@ class AzureBatch(BackendBase):
             thing["created"] = str(created)
             thing["elapsed"] = str(elapsed)
 
-        lb = ReportBuilder(self.config, self.store, client=None)
+        lb = ReportBuilder(self.config, self.store)
         text, rows = lb.build_formatted_table(things, columns)
         console.print(text)
 
@@ -277,6 +277,12 @@ class AzureBatch(BackendBase):
 
             self.print_things(pools, ["id", "state", "created", "elapsed"])
 
+    def make_batch_job_id(self, ws_name, job_id):
+        # qualify job_id with store_name and ws_name to minimize duplicate job names
+        store_name = self.config.get("store")
+        name = "{}__{}__{}".format(store_name, ws_name, job_id)
+        return name
+        
     def submit_job(self, job_id, job_runs, workspace, compute_def, resume_name, 
             repeat_count, using_hp, runs_by_box, experiment, snapshot_dir, controller_scripts, args):
         '''
@@ -324,7 +330,7 @@ class AzureBatch(BackendBase):
                 fb.feedback("{}/{}".format(workspace, run_name))
 
             node_id = "node" + str(n)
-            batch_job_id = "{}_{}".format(ws_name, job_id)
+            batch_job_id = self.make_batch_job_id(ws_name, job_id)
 
             service_info_by_node[node_id] = {"workspace": ws_name, "job_id": job_id, "batch_job_id": batch_job_id, 
                 "node_id": node_id, "task_index": n, "merge_batch_logs": merge_batch_logs}
@@ -413,9 +419,8 @@ class AzureBatch(BackendBase):
         self.auto_pool = auto_pool
         self.description = description
 
-        # must now qualify job_id with ws_name to prevent duplicate job names
         workspace = args["workspace"]
-        batch_job_id = workspace + "_" + job_id
+        batch_job_id = self.make_batch_job_id(workspace, job_id)
 
         self.batch_job_id = batch_job_id
         self.pool_id = "_pool_{}_{}_".format(workspace, job_id)
@@ -519,7 +524,7 @@ class AzureBatch(BackendBase):
 
         # XT always has exactly 1 task running on each node (xt controller), so
         # we can reply on task[x] running on node index x
-        batch_job_name = ws_name + "_" + job_id
+        batch_job_name = self.make_batch_job_id(ws_name, job_id)
         task_id = "task" + str(node_index)
         task = self.batch_client.task.get(batch_job_name, task_id)
 
@@ -938,7 +943,7 @@ class AzureBatch(BackendBase):
         # terminate the TASK
         #console.print("canceling: job={}, node_index={}, run_names={}".format(job_id, node_index, full_run_names))
 
-        batch_job_name = ws_name + "_" + job_id
+        batch_job_name = self.make_batch_job_id(ws_name, job_id)
         task = self.batch_client.task.get(batch_job_name, "task" + str(node_index))
         before_status = str(task.state)
         #console.print("task.state=", task.state)
@@ -1377,10 +1382,10 @@ class AzureBatch(BackendBase):
             # try to first read from job storage (if task has completed)
             node_index = utils.node_index(node_id)
             job_path = "node-{}/after/service_logs/{}".format(node_index, file_path)
-            if self.store.does_job_file_exist(job_id, job_path):
+            if self.store.does_job_file_exist(ws_name, job_id, job_path):
                 console.print("==> reading log from job storage...")
 
-                new_text = self.store.read_job_file(job_id, job_path)
+                new_text = self.store.read_job_file(workspace, job_id, job_path)
                 batch_status = "completed"
                 simple_status = "completed"
                 got_text = True
