@@ -273,6 +273,7 @@ class CmdCore():
         context.update_run_stats = self.config.get("mongo", "update-run-stats")
         context.add_log_records = self.config.get("mongo", "add-log-records")
         context.use_simple_scheduling = self.config.get("mongo", "use-simple-scheduling")
+        context.buffer_metrics = self.config.get("mongo", "buffer-metrics")
 
         # simple scheduling
         total_run_count = args["total_run_count"]
@@ -670,7 +671,7 @@ class CmdCore():
         status_list = status_text.split("\n")[0:-1]
 
         # create helper for filtering runs to show
-        builder = ReportBuilder(self.config, self.store, self)
+        builder = ReportBuilder(self.config, self.store)
         status = ""
 
         if status_list:
