@@ -6,6 +6,7 @@
 import sys
 import os
 import json
+from azureml.core import workspace
 import rpyc
 import copy
 import time
@@ -60,23 +61,6 @@ queue_lock = Lock()
 runs_lock = Lock()            
 rundir_lock = Lock()
 
-# def wait_for_process_exit(runner, pipe_thread, process, run_info):
-#     '''
-#     we use a separate thread to monitor for abnormal process exit that is 
-#     not triggered in our "read_from_pipe" thread.
-#     '''
-#     while True:
-#         time.sleep(1)
-#         if process.poll():
-#             break
-    
-#     # run post-processing
-
-#     # kill pipe_thread, if still running
-#     #pipe_thread.kill()
-#     print("wait_for_process_exit THREAD EXITING...")
-#     runner.exit_handler(run_info, called_from_thread_watcher=True)
-    
 def read_from_pipe(pipe, run_info, runner, stdout_is_text):
 
     # expand any "~" in path
@@ -141,6 +125,7 @@ class XTController(rpyc.Service):
         self.idle_check_count = 0
         self.workspace = None
         self.mri = None
+        self.node_restart = False
 
         # if previous controller log file exists, remove it
         fn = os.path.expanduser(constants.FN_CONTROLLER_EVENTS)
@@ -159,7 +144,6 @@ class XTController(rpyc.Service):
         self.runs = {}       # all runs that we know about (queued, spawing, running, or completed)
         self.queue = []      # runs that are waiting to run (due to concurrent)
         self.mirror_workers = []
-        self.restart_is_present = True
         self.running_jobs = {}
         self.next_run_index = 0
 
@@ -284,8 +268,21 @@ class XTController(rpyc.Service):
         self.store = store
         mongo = store.mongo
 
-        # log NODE_START to job
-        store.log_job_event(self.workspace, self.job_id, "node_start", {"node_id": self.node_id, "run": parent_run_name})
+        # is this node restarting after a low-priority preemption (or a simulated one)?
+        node_path = "node-{}/node.log".format(utils.node_index(self.node_id))
+
+        if store.does_job_file_exist(self.workspace, self.job_id, node_path):
+            self.node_restart = True
+            text = self.make_event_text("restarted")
+            store.append_job_file(self, workspace, self.job_id, node_path, text)
+            print("=========> NODE RESTART DETECTED <==============")
+        else:
+            text = self.make_event_text("started")
+            store.create_job_file(self.workspace, self.job_id, node_path, text)
+
+        # log NODE_START / NODE_RESTART to job
+        start_name = "node_restart" if self.node_restart else "node_start"
+        store.log_job_event(self.workspace, self.job_id, start_name, {"node_id": self.node_id, "run": parent_run_name})
 
         self.use_simple_scheduling = self.mrc_context.use_simple_scheduling
 
@@ -310,6 +307,11 @@ class XTController(rpyc.Service):
         # queue up first job
         self.queue_job_core(context, cmd_parts, aml_run=aml_run)
 
+    def make_event_text(self, event_name, dd={}):
+        nd = {"time": utils.get_time(), "event": event_name, "data": dd}
+        text = json.dumps(nd)
+        return text
+
     def write_connect_info_to_job(self, store, job_id):
 
         if os.getenv("PHILLY_CONTAINER_PORT_RANGE_START"):
@@ -543,12 +545,10 @@ class XTController(rpyc.Service):
 
             if entry:
                 run_index = entry["run_index"]
-                child_restarting = (entry["status"] == constants.WAITING)
-
                 log_info("==> running INDEX", "{}/{}".format(run_index, context.total_run_count-1))
 
                 # yes: CREATE CHILD
-                self.run_template(run_info, run_index, entry, child_restarting)
+                self.run_template(run_info, run_index, entry)
 
                 # insert back into queue
                 with queue_lock:
@@ -575,7 +575,7 @@ class XTController(rpyc.Service):
         with runs_lock:
             self.runs[key] = run_info
 
-    def run_template(self, run_info, run_index, entry, child_restarting):
+    def run_template(self, run_info, run_index, entry):
         run_name = run_info.run_name
 
         # ensure PARENT has a rundir (so it can log its own output)
@@ -593,7 +593,7 @@ class XTController(rpyc.Service):
         store.log_run_event(context.ws, run_name, "status-change", {"status": "spawning"})   # , is_aml=self.is_aml)
 
         # spawn child run from template
-        child_info = self.spawn_child(run_info, run_index, entry, child_restarting)
+        child_info = self.spawn_child(run_info, run_index, entry)
         child_name = child_info.run_name
 
         # add to runs
@@ -620,7 +620,7 @@ class XTController(rpyc.Service):
             console.print("xt controller - scheduling shutdown..")
             self.shutdown_requested = True
 
-    def spawn_child(self, parent, run_index, entry, child_restarting):
+    def spawn_child(self, parent, run_index, entry):
         spawn_start = time.time()
 
         # create a CLONE of template as a child run
@@ -630,7 +630,6 @@ class XTController(rpyc.Service):
         context = copy.copy(parent.context)
         context.repeat = None
         context.is_parent = False
-        context.restart = child_restarting
 
         #debug_break()
 
@@ -1112,14 +1111,6 @@ class XTController(rpyc.Service):
             script_args = None
             fn_script = None
 
-        # log run STARTED event 
-        if context.restart:
-            log_title("CHILD RESTART detected")
-
-        start_event_name = "restarted" if context.restart else "started"
-        store.log_run_event(context.ws, run_name, start_event_name, {})   # , is_aml=self.is_aml)
-        #prep_script = run_info.prep_script  
-
         exper_name = context.exper_name
 
         # local function
@@ -1222,6 +1213,49 @@ class XTController(rpyc.Service):
         dd = {"node_id": self.node_id, "run": run_name, "restart": context.restart}
         store.log_job_event(self.workspace, self.job_id, "start_run", dd)
 
+        # start a MIRROR thread to copy files to grok server
+        if context.mirror_dest and context.mirror_files and (context.grok_server or context.mirror_dest == "storage"):
+            log_info("mirror_dest=", context.mirror_dest)
+            log_info("mirror_files=", context.mirror_files)
+
+            self.start_mirror_worker(store, run_info, rundir, run_name, context)
+
+        # tell mongo JOBS that this job has a new run
+        console.print("calling MONGO job_run_start: job_id={}".format(job_id))
+        mongo = store.get_mongo()
+        if mongo:
+            # tell mongo RUNS that this run has started
+            console.print("calling MONGO run_start: ws={}, run_name={}".format(context.ws, run_name))
+            run_restarted = mongo.run_start(context.ws, run_name)
+            context.restart = run_restarted
+
+            if not run_restarted:
+                # don't count this run if it has been restarted
+                mongo.job_run_start(context.ws, job_id)
+
+        # log run STARTED event 
+        if context.restart:
+            log_title("CHILD RESTART detected")
+
+        start_event_name = "restarted" if context.restart else "started"
+        store.log_run_event(context.ws, run_name, start_event_name, {})   # , is_aml=self.is_aml)
+        #prep_script = run_info.prep_script  
+
+        if not job_id in self.running_jobs:
+            # this is the first run of this job on this node
+            self.running_jobs[job_id] = True
+
+            # tell mongo the first time this job node starts running
+            if mongo and not self.node_restart:
+                console.print("calling MONGO job_node_start: job_id={}".format(job_id))
+                mongo.job_node_start(self.workspace, job_id)
+
+        # don't start run until above logging has completed
+        self.start_run_now(rundir, parts, cwd, fn_script, child_env, stdout_is_text, bufsize, run_info)
+
+        return True
+
+    def start_run_now(self, rundir, parts, cwd, fn_script, child_env, stdout_is_text, bufsize, run_info):
         if pc_utils.is_windows():
             # target must be a fully qualified name to work reliably
             fq = os.path.join(rundir, parts[0])
@@ -1239,14 +1273,16 @@ class XTController(rpyc.Service):
             log_info("script", file_utils.read_text_file(fn_script))
 
             p = process = subprocess.Popen(parts, cwd=cwd, startupinfo=startupinfo, 
-                stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=child_env, universal_newlines=stdout_is_text, bufsize=bufsize)
+                stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=child_env, 
+                universal_newlines=stdout_is_text, bufsize=bufsize)
         else:
             log_info("cwd", cwd)
             log_info("parts", parts)
             log_info("script", file_utils.read_text_file(fn_script))
 
             p = process = subprocess.Popen(parts, cwd=cwd, 
-                stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=child_env, universal_newlines=stdout_is_text, bufsize=bufsize)
+                stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=child_env, 
+                universal_newlines=stdout_is_text, bufsize=bufsize)
 
         with run_info.lock:
             run_info.set_process(process)
@@ -1255,36 +1291,8 @@ class XTController(rpyc.Service):
         stdout_thread = Thread(target=read_from_pipe, args=(process.stdout, run_info, self, stdout_is_text))
         stdout_thread.start()
 
-        # start a MIRROR thread to copy files to grok server
-        if context.mirror_dest and context.mirror_files and (context.grok_server or context.mirror_dest == "storage"):
-            log_info("mirror_dest=", context.mirror_dest)
-            log_info("mirror_files=", context.mirror_files)
-
-            self.start_mirror_worker(store, run_info, rundir, run_name, context)
-
         log_info("process created", p)
 
-        # tell mongo JOBS that this job has a new run
-        console.diag("calling MONGO job_run_start: job_id={}".format(job_id))
-        mongo = store.get_mongo()
-        if mongo:
-            mongo.job_run_start(context.ws, job_id)
-
-            # tell mongo RUNS that this run has started
-            console.diag("calling MONGO run_start: ws={}, run_name={}".format(context.ws, run_name))
-            mongo.run_start(context.ws, run_name)
-
-        if not job_id in self.running_jobs:
-            # this is the first run of this job on this node
-            self.running_jobs[job_id] = True
-
-            if mongo and not context.restart:
-                # tell mongo the first time this job node starts running
-                console.diag("calling MONGO job_node_start: job_id={}".format(job_id))
-                mongo.job_node_start(context.ws, job_id)
-
-        return True
-
     def start_mirror_worker(self, store, run_info, rundir, run_name, context):
         console.print("starting a MIRROR thread: grok={}, mirror-dest={}, mirror-path={}".format(context.grok_server, 
             context.mirror_dest, context.mirror_files))
