@@ -247,7 +247,7 @@ class XTConfig():
             # is this target a box name?
             box_info = self.get("boxes", target_name, suppress_warning=True)
             if not box_info:
-                errors.config_error("target '{}' must be defined in the [compute-targets] section of XT config file (or be box name)".format(target_name))
+                errors.config_error("target '{}' must be defined in the [compute-targets] section of XT config file (or be a box name)".format(target_name))
             # make box look like a target
             compute_def = {"service": "pool", "boxes": [target_name]}
 
