@@ -13,6 +13,7 @@ import json
 import numpy as np
 import yaml
 
+from xtlib import utils
 from xtlib import console
 from xtlib import search_helper
 from xtlib.hparams import hp_helper
@@ -331,6 +332,7 @@ class SearchHistory():
     def init_xtstore(self):
         if self.xtstore is None:
             xt_config = get_merged_config(create_if_needed=False)
+            self.xt_config = xt_config
             self.xtstore = Store(config=xt_config)
 
     def load_hp_definitions(self):
@@ -522,28 +524,55 @@ class SearchHistory():
         ys = np.zeros((num_runs))
         ds = np.zeros((num_runs))
         x = 0
-        # Write the header.
-        headers = ' '
+
+        #build headers and rows for report
+        hp_names = []
         for hp in self.hparams:
             if hp.in_hp_section and hp.has_multiple_values:
-                width = max(MIN_COL_WID, len(hp.name))
-                format_string = ' {' + ':{}s'.format(width) + '}'
-                headers += format_string.format(hp.name)
-        headers += 'BEST FOR   ' + 'EST HPMAX  ' + 'RUNS  ' + 'BEST'
-        print(headers)
+                hp_names.append(hp.name)
+
+        fixed_names = ['BEST FOR', 'EST HPMAX', 'RUNS', 'BEST']
+        avail_cols = hp_names + fixed_names
+
+        records = []
         for trophy in self.best_runset_trophies:
             trophy.runset.runs.sort(key=get_neg_run_mean)
             best = '{}'.format(trophy.runset.runs[0].run_name)  # To show only the best run for each runset.
-            print(" {}{:3d} runs   {:9.6f}  {:3d}   {}".format(trophy.runset.report_configuration_string(),
-                trophy.num_runs, trophy.runset.post_mean, len(trophy.runset.runs), best))
+
+            # print(" {}{:3d} runs   {:9.6f}  {:3d}   {}".format(trophy.runset.report_configuration_string(),
+            #     trophy.num_runs, trophy.runset.post_mean, len(trophy.runset.runs), best))
+
+            hp_values = trophy.runset.report_configuration_string().split()
+            hd = {header:utils.make_numeric_if_possible(value) for header, value in zip(hp_names, hp_values)}
+
+            fixed_values = [trophy.num_runs, trophy.runset.post_mean, len(trophy.runset.runs), best]
+            fd = {name: value for name, value in zip(fixed_names, fixed_values)}
+
+            # merge fd into hd
+            hd.update(fd)
+            records.append(hd)
+
+            # calc data for plot
             y = trophy.runset.post_mean
             d = math.sqrt(trophy.runset.post_var)
+
             for i in range(trophy.num_runs):
                 xs[x] = x + 1
                 ys[x] = y
                 ds[x] = d
                 x += 1
-        print(headers)
+
+        from xtlib.report_builder import ReportBuilder   
+        builder = ReportBuilder(self.xt_config, self.xtstore)
+
+        # build report (borrow config file settings from run-reports)
+        text, row_count = builder.build_formatted_table(records, avail_cols=avail_cols, col_list=avail_cols, 
+            report_type="run-reports", max_col_width=50)
+
+        # print report
+        print()
+        print(text)
+
         print("  History of the best hyperparameter combination after each of the {} completed runs.".format(len(self.runs)))
 
         # Plot the search progress.
