@@ -42,7 +42,7 @@ class HParamSearch():
                 text = store.read_experiment_file(context.ws, context.dest_name, context.hp_config)
             else:    
                 # assume sweeps file is at the job level
-                text = store.read_job_file(context.dest_name, context.hp_config)
+                text = store.read_job_file(context.ws, context.dest_name, context.hp_config)
 
             arg_dict = self.generate_hparam_set(context.hp_config, text, child_name, cmd_parts, store, context)
 
@@ -197,7 +197,8 @@ class HParamSearch():
         fn_cache = "hparams_search.cache"
 
         # get all completed runs from MONGO 
-        new_runs = store.get_all_runs(context.aggregate_dest, context.ws, context.dest_name, filter_dict, fields_dict, use_cache=False)
+        new_runs = store.get_all_runs(context.aggregate_dest, context.ws, context.dest_name, filter_dict, fields_dict,
+            use_cache=False)
 
         # merge new runs with previous
         print("get_completed_runs: run_history={:,}, new_runs={:,}".format(len(self.run_history), len(new_runs)))
@@ -209,7 +210,7 @@ class HParamSearch():
         self.new_run_count = len(new_runs)
 
         if self.new_run_count:
-            end_ids = [run["end_id"] for run in new_runs]
+            end_ids = [run["end_id"] for run in new_runs if "end_id" in run]
             self.end_id = max(end_ids)
 
         return runs
