@@ -285,7 +285,7 @@ class ImplCompute(ImplBase):
                         node_id = utils.node_id(node_index)
 
                         # set new context
-                        service_node_info = service_info_by_node[node_index]
+                        service_node_info = service_info_by_node[node_id]
                         start_offset = offset_by_node[node_id] if node_id in offset_by_node else 0
                         first_text_of_stream = not start_offset
                         console.print("==> switching to: node{}".format(node_index))
@@ -584,7 +584,7 @@ class ImplCompute(ImplBase):
         run_names, actual_ws = run_helper.parse_run_list(self.store, workspace, run_names)
 
         mongo = self.store.get_mongo()
-        fields_dict = {"box_name": 1, "compute": 1, "ws": 1, "run_name": 1}
+        fields_dict = {"box_name": 1, "compute": 1, "ws_name": 1, "run_name": 1}
 
         filter_dict = {}
         filter_dict["run_name"] = {"$in": run_names}
@@ -617,7 +617,7 @@ class ImplCompute(ImplBase):
         for box_name, runs in runs_by_box.items():
             first_run = runs[0]
 
-            ws_name = first_run["ws_name"]
+            ws_name = workspace    # first_run["ws_name"]
             # = [ws_name + "/" + run["run_name"] for run in runs]
             run_names = [run["run_name"] for run in runs]
             compute = first_run["compute"] if "compute" in first_run else "pool"    # some legacy support
@@ -630,28 +630,29 @@ class ImplCompute(ImplBase):
         self.print_cancel_all_results(cancel_results_by_box)
 
     #---- CANCEL JOB command ----
-    @argument("job-id", help="the name of the job to cancel")
-    @option("workspace", default="$general.workspace", help="the workspace that contains the job to be cancelled")
-    @example(task="cancel all runs in job3125", text="xt cancel job job3125")
-    @command(kwgroup="cancel", kwhelp="cancels active runs, specified by name, job, or target", help="cancels the specified job and its active or queued runs")
-    def cancel_job(self, job_id, workspace):
+    @argument("job-names", type="str_list", help="the names of the jobs to be cancelled")
+    @option("workspace", default="$general.workspace", help="the workspace that contains the jobs to be cancelled")
+    @example(task="cancel jobs job3 and job15", text="xt cancel job job3, job15")
+    @command(kwgroup="cancel", kwhelp="cancels active runs, specified by name, job, or target", help="cancels the specified jobs and their active or queued runs")
+    def cancel_job(self, job_names, workspace):
 
-        console.print("cancelling {}:".format(job_id))
+        for job_id in job_names:
+            console.print("cancelling {}:".format(job_id))
 
-        service_job_info, service_info_by_node, backend = job_helper.get_service_job_info(self.store, self.core, workspace, job_id)
-        result_by_node = backend.cancel_job(service_job_info, service_info_by_node)
+            service_job_info, service_info_by_node, backend = job_helper.get_service_job_info(self.store, self.core, workspace, job_id)
+            result_by_node = backend.cancel_job(service_job_info, service_info_by_node)
 
-        for node_id, result in result_by_node.items():
-            console.print("  {}: {}".format(node_id, result))
+            for node_id, result in result_by_node.items():
+                console.print("  {}: {}".format(node_id, result))
 
-        # wrap up runs associated with each node
-        wrapup_count = 0
+            # wrap up runs associated with each node
+            wrapup_count = 0
 
-        for node_id, result in result_by_node.items():
-            runs = run_helper.wrapup_runs_by_node(self.store, workspace, job_id, node_id, result)
-            wrapup_count += len(runs)
+            for node_id, result in result_by_node.items():
+                runs = run_helper.wrapup_runs_by_node(self.store, workspace, job_id, node_id, result)
+                wrapup_count += len(runs)
 
-        console.print("  runs wrapped-up: {}".format(wrapup_count))
+            console.print("  runs wrapped-up: {}".format(wrapup_count))
 
     #---- CANCEL ALL command ----
     @argument("target", help="the name of the compute target whose runs will be cancelled")
