@@ -458,6 +458,74 @@ class ImplUtilities(ImplBase):
         with open(fn, "w") as newfile:
             newfile.write(text)
 
+    #---- CHANGE CONFIG command ----
+    @argument(name="property", type=str, help="specifies the (dotted) name of the xt config property to be changed")
+    @argument(name="value", type=str, required=False, help="specifies the new value for the property to be changed")
+    @example(task="change the default workspace to 'ws5' in the user's local config file", 
+        text="xt change config general.workspace ws5")
+    @see_also("XT Config File", "xt_config_file")
+    @command(help="changes the value of a property in the user's local XT config file")
+    def change_config(self, property, value):
+        fn = "xt_config.yaml"
+        text = file_utils.read_text_file(fn)
+        lines = text.split("\n")
+
+        props_stack = []
+        current_property = ""
+        found = False
+
+        for line_index, line in enumerate(lines):
+            # find first non-whitespace char in line
+            stripped = line.lstrip()
+            if stripped:
+                indent = len(line) - len(stripped)
+
+                if not stripped.startswith("#") and ":" in stripped:
+                    # it is a property assignment
+                    left, right = stripped.split(":", 1)
+                    prop_name = left.strip()
+                    prop_value = right.split()[0] if right else ""
+
+                    # apply to props_stack
+                    new_entry = (indent, prop_name)
+
+                    if props_stack:
+                        # remove any obsolete entries
+                        top_indent, top_name = props_stack[-1]
+                        while indent <= top_indent:
+                            props_stack.pop()
+                            current_property = current_property.rsplit(".")[0] if props_stack else ""
+                            if not props_stack:
+                                break
+                            top_indent, top_name = props_stack[-1]
+
+                    props_stack.append(new_entry)
+                    if current_property:
+                        current_property += "." + prop_name 
+                    else:
+                        current_property = prop_name
+
+                    #console.print("  " + current_property)
+
+                    if current_property == property:
+                        found = True
+                        if value:
+                            line = line.replace(prop_value, value)
+                            lines[line_index] = line
+                        break
+
+        if found:
+            if value:
+                text = "\n".join(lines)
+                file_utils.write_text_file(fn, text)
+
+                console.print("{} set to: {}".format(property, value))
+            else:
+                console.print("{}: {}".format(property, prop_value))
+        else:
+            console.print("property not found: {}".format(property))
+
+
     #---- CONFIG command ----
     #@flag(name="local", help="edit the config file in the current directory")
     @option(name="response", help="the response to use if a new config file needs to be created")
