@@ -15,6 +15,15 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import file_utils
 from xtlib import store_utils
+from xtlib import run_helper
+
+job_info_props = {"_id": 1, "compute": 1, "concurrent": 1, "exper_name": 1, "hold": 1, "job_id": 1, "job_num": 1,
+    "job_guid": 1, "job_secret": 1, "node_count": 1, "primary_metric": 1, "run_count": 1, "repeat": 1, 
+    "schedule": 1, "search_type": 1, "search_style": 1, "username": 1, "xt_cmd": 1, "started": 1,
+    "pool_info": 1, "service_job_info": 1, "ws_name": 1}
+
+job_stats_props = {"job_status": 1, "completed_runs": 1, "error_runs": 1, "running_nodes": 1, "total_runs": 1, "tags": 1}
+
 
 def is_job_id(name):
     # compatible with version 1 (prefix-jobNNNN) and version 2 (jobNNNN)
@@ -47,8 +56,24 @@ def set_job_tags(store, mongo, name_list, tag_list, workspace, fd, clear):
     job_list, actual_ws = expand_job_list(store, mongo, workspace, name_list, can_mix=True)
 
     if job_list:
-        for job in job_list:
-            mongo.update_job_info(workspace, job, fd, clear=clear, upsert=False)   
+        # for job in job_list:
+        #     mongo.update_job_info(workspace, job, fd, clear=clear, upsert=False)   
+
+        if len(job_list)==1 and job_list[0] == "*":
+            # update ALL records in ws
+            all_ws_jobs = mongo.get_info_for_jobs(workspace, {"ws_name": workspace}, {"_id": 1})
+            job_list = [job["_id"] for job in all_ws_jobs]
+        else:
+            # convert simple job_name into id required by storage version
+            job_list = [store_utils.make_id(workspace, job) for job in job_list]
+
+            matched_count = mongo.update_job_stats_from_list(workspace, job_list, fd, clear=clear, upsert=False)
+
+        if clear:
+            console.print("{} jobs, tags cleared: {}".format(matched_count, tag_list))
+        else:
+            console.print("{} jobs, tags set: {}".format(matched_count, tag_list))
+
     else:
         console.print("no matching jobs found")
 
@@ -58,7 +83,8 @@ def list_job_tags(store, mongo, ws_name, name_list, tag_list):
 
     if job_list:
         filter_dict = {"job_id": {"$in": job_list}}
-        fields_dict = {"tags": 1}
+        #fields_dict = {"tags": 1}
+        fields_dict = {"job_stats": 1}
 
         records = mongo.get_info_for_jobs(ws_name, filter_dict, fields_dict)
         for record in records:
@@ -117,6 +143,7 @@ def build_job_filter_dict(ws_name, job_list, user_to_actual, builder, workspace,
     tags_any = args["tags_any"]
     if tags_any:
         fany_list = []
+        
         for tag in tags_any:
             f = {"tags." + tag: {"$exists": True}}
             fany_list.append(f)
@@ -134,7 +161,7 @@ def get_job_property_dicts():
         "repeat": "repeat", "search": "search_type", "service_type": "pool_info.service", 
         "sku": "pool_info.sku", "runs": "run_count", "search_style": "search_style", "started": "started", "target": "compute", "username": "username", 
         "vc": "pool_info.vc", "vm_size": "pool_info.vm-size", "workspace": "ws_name",  
-        "tags": "tags",
+        "hparams": "hparams", "tags": "tags",
 
         # dynamic properties (get updated at various stages of job)
         "status": "job_status", "running_nodes": "running_nodes", 
@@ -143,6 +170,7 @@ def get_job_property_dicts():
         }
 
     std_cols_desc = {
+        # the key here is the USER friendly name (not the physical name)
         "cluster": "the Philly Cluster that the job ran on",
         "docker": "the Docker environment specified for this job",
         "experiment": "the experiment associated with the job",
@@ -167,7 +195,7 @@ def get_job_property_dicts():
         "workspace": "the workspace associated with the job",
 
         # dynamic properties (get updated at various stages of job)
-        "job_status": "one of: submitted, running, completed",
+        "status": "one of: submitted, running, completed",
         "running_nodes": "the number of nodes in the running state",
         "running_runs": "the number of this job's runs that are current running",
         "completed_runs": "the number of runs that have completed (with or without errors)",
@@ -181,7 +209,7 @@ def get_list_jobs_records(store, config, args):
     job_list = args["job_list"]
     pool = args["target"]
 
-    if job_list:
+    if job_list and store_utils.STORAGE_FORMAT == "1":
         # only use workspace if it was explictly set  
         workspace = None   
         explict = qfe.get_explicit_options()
@@ -197,7 +225,7 @@ def get_list_jobs_records(store, config, args):
     user_to_actual, std_cols_desc = get_job_property_dicts()        
     actual_to_user = {value: key for key, value in user_to_actual.items()}
 
-    builder = ReportBuilder(config, store, client=None)
+    builder = ReportBuilder(config, store)
 
     # get list of specified jobs
     mongo = store.get_mongo()
@@ -211,20 +239,33 @@ def get_list_jobs_records(store, config, args):
     records, using_default_last, last = builder.get_mongo_records(mongo, filter_dict, workspace, "jobs", actual_to_user, args=args)
     return records, using_default_last, last, user_to_actual, builder
 
-def list_jobs(store, config, args):
+
+
+def list_jobs(store, config, args, compare=False):
     available = args["available"]
 
+    if compare:
+        args["columns"] = ["job", "hparams.*"]
+
     records, using_default_last, last, user_to_actual, builder \
         = get_list_jobs_records(store, config, args)
 
     if available:
+        user_to_actual, std_cols_desc = get_job_property_dicts()        
+        hparams_cols = run_helper.extract_dotted_cols(records, "hparams")
         std_cols = list(user_to_actual.keys())
         tag_cols = extract_tag_cols(records)
-        lines = builder.available_cols_report("job", std_cols, std_cols_desc, tags_list=tag_cols)
+        lines = builder.available_cols_report("job", std_cols, std_cols_desc, hparams_list=hparams_cols, 
+            tags_list=tag_cols)
 
         for line in lines:
             console.print(line)
-    else:            
+    else:    
+        if compare:
+            # filter cols to those that are different
+            diff_cols = run_helper.get_different_cols(records)
+            args["columns"] = diff_cols 
+
         avail_list = list(user_to_actual.keys())
         lines, row_count, was_exported = builder.build_report(records, report_type="job-reports", args=args)
 
