@@ -11,6 +11,44 @@ import socket
 from xtlib import console
 from xtlib import constants
 
+# ESCAPE CODES FOR SCREEN
+#reference: https://www.codeproject.com/Tips/5255355/How-to-Put-Color-on-Windows-Console
+
+# special actions
+CLEAR_LINE = "\033[K"
+CURSOR_UP = "\033[F"
+
+# foreground colors
+NORMAL = "\033[0m"
+BOLD = "\033[1m"
+
+UNDERLINE = "\033[1;4m"
+NOUNDERLINE = "\033[1;24m"
+NEGATIVE = "\033[1;7m"
+POSITIVE = "\033[1;27m"
+
+BLACK = "\033[1;30m"
+RED = "\033[1;31m"
+GREEN = "\033[1;32m"
+YELLOW = "\033[1;33m"
+BLUE = "\033[1;34m"
+MAGENTA = "\033[1;35m"
+CYAN = "\033[1;36m"
+GRAY = "\033[0;37m"         # default color
+
+COLORS = [BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, GRAY]
+
+LIGHTRED = "\033[1;91m"
+LIGHTGREEN = "\033[1;92m"
+LIGHTYELLOW = "\033[1;93m"
+LIGHTBLUE = "\033[1;94m"
+LIGHTMAGENTA = "\033[1;95m"
+LIGHTCYAN = "\033[1;96m"
+DARKGRAY = "\033[1;90m"
+WHITE = "\033[1;97m"
+
+LIGHT_COLORS = [LIGHTRED, LIGHTGREEN, LIGHTYELLOW, LIGHTBLUE, LIGHTMAGENTA, LIGHTCYAN, DARKGRAY, WHITE]
+
 def is_windows():   
     return os.name == "nt"
 
@@ -122,9 +160,7 @@ def input_with_default(prompt, default, dim_prompt=True, use_gray=False):
 
         if dim_prompt:
             enable_ansi_escape_chars_on_windows_10()
-            gray = "\033[0;37m"
-            darkgray = "\033[1;30m"
-            value = input(darkgray + prompt + gray)
+            value = input(DARKGRAY + prompt + GRAY)
         else:
             value = input(prompt)
 
@@ -145,13 +181,18 @@ def move_cursor_up(line_count, clear_lines=True):
     #     sys.stdout.write("\r\033[K") 
 
     for _ in range(line_count):
-        sys.stdout.write("\033[F") 
+        sys.stdout.write(CURSOR_UP) 
         if clear_lines:
             # clear line we just moved into
-            sys.stdout.write("\033[K") 
+            sys.stdout.write(CLEAR_LINE) 
+
+def highlight(text):
+    sys.stdout.write(WHITE)
+    print(text)
+    sys.stdout.write(NORMAL)
 
 def clear_line():
-    sys.stdout.write("\033[K") 
+    sys.stdout.write(CLEAR_LINE) 
 
 def wait_for_escape(checker, wait_time=1, check_time=.1):
     ''' 
