@@ -14,6 +14,8 @@ from xtlib import errors
 from xtlib import console
 from xtlib import constants
 from xtlib import run_helper
+from xtlib import job_helper
+from xtlib import store_utils
 
 PRESMOOTH = "_PRE-SMOOTH_"
 ERR = "_ERR_"
@@ -22,13 +24,15 @@ MIN = "_MIN_"
 MAX = "_MAX_"
 
 class PlotBuilder():
-    def __init__(self, run_names, col_names, x_col, layout, break_on, title, show_legend, plot_titles,
+    def __init__(self, store, run_names, col_names, x_col, layout, break_on, title, show_legend, plot_titles,
             legend_titles, smoothing_factor, plot_type, timeout,
             aggregate, shadow_type, shadow_alpha, run_log_records, style, show_toolbar, max_traces,
             group_by, error_bars, show_plot, save_to, x_label, colors, color_map, color_steps, legend_args, 
             x_min, x_max, y_min, y_max, alpha, line_size, timebase, x_format, y_format, 
-            x_share, y_share, background_color, grid_color, clean, width, height, plot_args):
+            x_share, y_share, background_color, grid_color, clean, width, height, highlight, color_highlight,
+            legend_suffix, plot_args):
         
+        self.store = store
         self.group_names = None
         self.run_names = run_names
         self.col_names = col_names
@@ -72,6 +76,11 @@ class PlotBuilder():
         self.clean = clean
         self.width = width 
         self.height = height
+        self.highlight = highlight
+        self.color_highlight = color_highlight
+        self.legend_suffix = legend_suffix
+        self.group_info = None
+        self.ws_name = None
 
         if colors:
             self.colors = colors
@@ -142,7 +151,11 @@ class PlotBuilder():
         data_frames_by_cols = {}
         explicit_options = qfe.get_explicit_options()
         missing_col_runs = 0
+        missing_metrics = {}
 
+        self.group_info = {}
+        
+        # for each RUN
         for i, record in enumerate(self.run_log_records):
             # extract metrics for this run
             run = record["run_name"]
@@ -153,10 +166,27 @@ class PlotBuilder():
             experiment = record["exper_name"]
             workspace = record["ws_name"]
 
+            # remember for later use
+            self.ws_name = workspace
+
             group_value = self.get_group_value(self.group_by, run, node, job, experiment, workspace)
             if not group_value in group_order_dict:
                 group_order_dict[group_value] = 1
 
+            info_record = {"run": run, "job": job, "node": node, "experiment": experiment}
+
+            # fixup any aggregated items as "*"
+            if not self.group_by in ["run"]:
+                info_record["run"] = "run*"
+
+            if not self.group_by in ["run", "node"]:
+                info_record["node"] = "node*"
+
+            if not self.group_by in ["run", "node", "job"]:
+                info_record["job"] = "job*"
+
+            self.group_info[group_value] = info_record
+
             search_style = utils.safe_value(record, "search_style")
             if search_style and search_style != "single":
                 # parent run with children - skip it
@@ -172,7 +202,12 @@ class PlotBuilder():
             if self.x_col:
                 # ensure x_col is in this run
                 if not self.is_col_in_sets(metric_sets, self.x_col):
-                    console.print("warning: {} (missing metric: {})".format(run, self.x_col))
+
+                    # add to missing_metrics
+                    if not run in missing_metrics:
+                        missing_metrics[run] = {}
+                    missing_metrics[run][self.x_col] = True
+
                     missing_col_runs += 1
                     #continue    # SKIP this run
             else:
@@ -185,7 +220,12 @@ class PlotBuilder():
 
                 for col in self.col_names:
                     if not self.is_col_in_sets(metric_sets, col):
-                        console.print("warning: {} (missing metric: {})".format(run, col))
+
+                        # add to missing_metrics
+                        if not run in missing_metrics:
+                            missing_metrics[run] = {}
+                        missing_metrics[run][col] = True
+
                         found_all = False
                         break
 
@@ -237,6 +277,11 @@ class PlotBuilder():
 
             pp_run_names.append(run)
 
+        # report on runs missing metrics
+        for run, md in missing_metrics.items():
+            cols = ", ".join(md.keys())
+            console.print("warning: {} (missing metrics: {})".format(run, cols))
+
         if no_metrics:
             console.print("\nnote: following runs currently have no logged metrics: \n    {}\n".format(", ".join(no_metrics)))
 
@@ -545,15 +590,15 @@ class PlotBuilder():
         runs_per_plot = 1 if break_on_groups else group_count
         cols_per_plot = 1 if break_on_cols else col_count
         
-        if runs_per_plot == 1:
-            plot_title = "$run"
-            legend_text = "$col"
-        elif cols_per_plot == 1:
+        if cols_per_plot == 1:
             plot_title = "$col"
-            legend_text = "$run"
+            legend_text = "$group"
+        elif runs_per_plot == 1:
+            plot_title = "$group"
+            legend_text = "$col"
         else:
             plot_title = None
-            legend_text = "$col ($run)"
+            legend_text = "$col ($group)"
 
         if not self.plot_titles and plot_title:
             self.plot_titles = [plot_title]
@@ -671,6 +716,8 @@ class PlotBuilder():
         # mgr = plt.get_current_fig_manager()
         # mgr.window.full_screen_toggle(True)
 
+        self.adjust_legend_fonts(plots)
+
         ax = plots[0]
 
         if self.x_col == "__time__":
@@ -697,6 +744,101 @@ class PlotBuilder():
         if self.show_plot:
             pylab.show()
 
+    def adjust_legend_fonts(self, plots):
+        import matplotlib as mpl
+
+        size = 10
+        normal_font = mpl.font_manager.FontProperties(size=size)
+        is_alive = False
+        alive_font = None
+        alive_color = None
+        group_keys = list(self.group_info.keys())
+
+        if self.highlight == "$alive":
+            # choose a font that support color_highlight
+            if self.color_highlight == "bold":
+                alive_font = mpl.font_manager.FontProperties(size=size, weight='bold')
+            elif self.color_highlight == "italic":
+                alive_font = mpl.font_manager.FontProperties(size=size, style='italic')
+            elif self.color_highlight:
+                alive_color = self.color_highlight
+
+        for ax in plots:
+            # adjust font styling for legends
+            legend = ax.get_legend()
+            if legend:
+                alive_dict = None
+
+                if self.highlight == "$alive":
+                    group_values = list(self.group_info.keys())
+                    alive_dict = self.get_alive_status(group_values)
+
+                for i, texts in enumerate(legend.get_texts()):
+
+                    if self.highlight == "$alive":
+                        key_index = i % len(group_keys)
+                        group_name = group_keys[key_index]
+                        is_alive = group_name in alive_dict
+
+                    if is_alive:
+                        if alive_color:
+                            texts.set_color(alive_color)
+                        else:
+                            texts.set_fontproperties(alive_font)
+                    else:
+                        texts.set_fontproperties(normal_font)
+
+    def get_alive_status(self, names):
+        alive_dict = {}
+        first_name = names[0]
+
+        if run_helper.is_run_name(first_name):
+
+            # are specified RUNS alive?
+            id_list = [store_utils.make_id(self.ws_name, name) for name in names]
+            filter_dict = {"_id": {"$in": id_list}}
+            fields_dict = {"_id": 1, "status": 1}
+            
+            records = self.store.mongo.get_info_for_runs(self.ws_name, filter_dict, fields_dict)
+            store_utils.simplify_records_id(records)
+
+            for record in records:
+                status = utils.safe_value(record, "status")
+                if status in ["queued", "running"]:
+                    run_name = record["_id"]
+                    alive_dict[run_name] = True
+
+        elif job_helper.is_job_id(first_name):
+            # are RUNS alive in specified JOBS
+            filter_dict = {"job_id": {"$in": names}}
+            fields_dict = {"job_id": 1, "status": 1}
+            
+            records = self.store.mongo.get_info_for_runs(self.ws_name, filter_dict, fields_dict)
+            store_utils.simplify_records_id(records)
+
+            for record in records:
+                status = utils.safe_value(record, "status")
+                if status in ["queued", "running"]:
+                    job_name = record["job_id"]
+                    alive_dict[job_name] = True
+
+
+        else:
+            # are RUNS alive in specified EXPERIMENTS
+            filter_dict = {"exper_name": {"$in": names}}
+            fields_dict = {"exper_name": 1, "status": 1}
+            
+            records = self.store.mongo.get_info_for_runs(self.ws_name, filter_dict, fields_dict)
+            store_utils.simplify_records_id(records)
+
+            for record in records:
+                status = utils.safe_value(record, "status")
+                if status in ["queued", "running"]:
+                    exper_name = record["exper_name"]
+                    alive_dict[exper_name] = True
+
+        return alive_dict
+
     def get_seaborn_color_map(self, name, n_colors=5):
         '''
         name: muted, xxx
@@ -754,20 +896,6 @@ class PlotBuilder():
 
         self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, plot_index, x_values=x, y_values=y, 
             color=color, alpha=self.alpha, use_y_label=True, err_values=err)
-
-    def add_plot_legend_and_title(self, ax, plot_index, line_index, group_name, col, use_y_label):
-        if self.plot_titles:
-            plot_title = self.plot_titles[plot_index % len(self.plot_titles)]
-            plot_title = self.fixup_text(plot_title, group_name, col)
-
-            ax.set_title(plot_title)
-
-        if self.show_legend:
-            if self.legend_args:
-                # pass legend args to legend object
-                ax.legend(**self.legend_args)
-            else:
-                ax.legend()
         
     def plot_inner(self, ax, group_name, col, x_col, x_label, line_index, plot_index, x_values, y_values, 
         color, alpha, use_y_label, y2_values=None, err_values=None):
@@ -790,7 +918,12 @@ class PlotBuilder():
 
         if use_y_label:
             line_title = self.legend_titles[line_index % len(self.legend_titles)]
+
+            if self.legend_suffix:
+                line_title += self.legend_suffix
+
             line_title = self.fixup_text(line_title, group_name, col)
+
         else:
             line_title = None
             
@@ -841,12 +974,41 @@ class PlotBuilder():
             errors.syntax_error("unknown plot type={}".format(self.plot_type))
 
         # this must be called AFTER plotting
-        self.add_plot_legend_and_title(ax, plot_index, line_index, group_name, col, use_y_label)
+        if self.plot_titles:
+            # set the title of the subplot
+            plot_title = self.plot_titles[plot_index % len(self.plot_titles)]
+            plot_title = self.fixup_text(plot_title, group_name, col)
+            ax.set_title(plot_title)
+
+        if self.show_legend:
+            if self.legend_args:
+                # pass legend args to legend object
+                ax.legend(**self.legend_args)
+            else:
+                ax.legend()
 
     def fixup_text(self, text, group_name, col):
-        text = text.replace("$job", group_name)
-        text = text.replace("$run", group_name)
+        info = self.group_info[group_name]
+
+        if "$job" in text:
+            job_name = info["job"]
+            text = text.replace("$job", job_name)
+
+        if "$run" in text:
+            run_name = info["run"]
+            text = text.replace("$run", run_name)
+
+        if "$node" in text:
+            node_name = info["node"]
+            text = text.replace("$node", node_name)
+        
+        if "$experiment" in text:
+            exper_name = info["experiment"]
+            text = text.replace("$experiment", exper_name)
+        
+        text = text.replace("$group", group_name)
         text = text.replace("$col", col)
+
         return text
 
     def is_col_in_sets(self, metric_sets, col_name):
