@@ -570,18 +570,11 @@ class Dispatcher():
         for opt in cmd_info["options"]:
             opt["hidden"] = not (opt["name"] in args_to_show)
 
-    def match(self, text, cmd):
-        '''
-        user can abbreviate down to 3 letters, but text must unambiguosly select
-        an keyword.
-        '''
-        return cmd == text or (len(text) >= 3 and cmd.startswith(text))
-
     def match_keyword(self, value, keywords):
         found = None
 
         for kw in keywords:
-            if self.match(value, kw):
+            if match(value, kw):
                 found = kw
                 break
         
@@ -789,7 +782,7 @@ class Dispatcher():
         matches = [value for value in values if name == value["name"]]
         if not matches:
             # look for abbreviated matches
-            matches = [value for value in values if self.match(name, value["name"])]
+            matches = [value for value in values if match(name, value["name"])]
             if len(matches) > 1:
                 match_names = [m["name"] for m in matches]
                 errors.syntax_error("abbreviated name '{}' results in multiple matches: {}" \
@@ -799,7 +792,7 @@ class Dispatcher():
         return found
 
     def list_item_by_value(self, value, values):
-        matches = [val for val in values if self.match(value, val)]
+        matches = [val for val in values if match(value, val)]
         if len(matches) > 1:
             errors.syntax_error("abbreviated value '{}' results in multiple matches: {}".format(value, ", ".join(matches)))
         found = matches[0] if matches else None
@@ -889,7 +882,7 @@ class Dispatcher():
             if arg_type == "cmd" and tok and not tok.startswith("-"):
                 # convert remaining tokens to a cmd_info
                 if tok:
-                    # if self.match(tok, "topics"):
+                    # if match(tok, "topics"):
                     #     cmd_info = {"name": "topics"}
                     #     tok = scanner.scan()
                     # else:
@@ -1397,6 +1390,10 @@ class Dispatcher():
                 value = self.parse_flag_value(name, value)
             else:
                 value = True
+        elif keywords:
+            value = tok
+            tok = scanner.scan()
+            value = self.match_keyword(value, keywords)
         else:
             # its a simple value
             value = tok
@@ -1422,4 +1419,13 @@ class Dispatcher():
             else:
                 errors.internal_error("Unsupported opt_type={}".format(opt_type))
         
-        return value, tok
\ No newline at end of file
+        return value, tok
+
+# flat functions
+def match(text, cmd):
+    '''
+    user can abbreviate down to 3 letters, but text must unambiguosly select
+    an keyword.
+    '''
+    return cmd == text or (len(text) >= 3 and cmd.startswith(text))
+
