@@ -6,6 +6,7 @@
 import os
 import json
 import time
+from xtlib.pc_utils import COLORS
 import arrow
 import datetime
 import logging
@@ -18,6 +19,7 @@ from xtlib.console import console
 from xtlib import qfe
 from xtlib import utils
 from xtlib import errors
+from xtlib import pc_utils
 from xtlib import constants
 
 class ReportBuilder():
@@ -200,7 +202,7 @@ class ReportBuilder():
             else:
                 col_dict[prefix + key] = 1
 
-    def flatten_records(self, records, sort_col, args):
+    def flatten_records(self, records, sort_col, need_alive, args):
         '''
         pull out the AVAILABLE or USER SPECIFIED columns, flattening nested props to their dotted names.
         '''
@@ -208,17 +210,21 @@ class ReportBuilder():
         # build avail col list based on final set of filtered records
         avail_cols, actual_cols, user_col_args  = self.get_avail_actual_user_cols(records, args)
 
+        # add often-needed cols for processing report
+        for col in ["status", "created", "started"]:
+            if not col in actual_cols:
+                actual_cols.append(col)
+
         # flatten each record's nested columns
         available = utils.safe_value(args, "available")
 
-        if available:
-            records = [self.extract_actual_cols(rec, avail_cols) for rec in records if rec]
-        else:
-            records = [self.extract_actual_cols(rec, actual_cols) for rec in records if rec]
+        add_if_missing_cols = ["queued", "duration"]
+        get_cols = avail_cols if available else actual_cols
 
+        records = [self.extract_actual_cols(rec, get_cols, add_if_missing_cols) for rec in records if rec]
         return records
 
-    def extract_actual_cols(self, record, actual_cols):
+    def extract_actual_cols(self, record, actual_cols, add_if_missing_cols=None):
         '''
         pull out the cols specified in actual_cols, flattening nested props to their dotted names.
         '''
@@ -245,6 +251,11 @@ class ReportBuilder():
                     value = record[actual_key]
                     new_record[actual_key] = value if value is not None else empty_value
     
+        if add_if_missing_cols:
+            for col in add_if_missing_cols:
+                if not col in new_record:
+                    new_record[col] = None
+
         return new_record
 
     def translate_record(self, record, actual_to_user):
@@ -289,7 +300,7 @@ class ReportBuilder():
         return first, last
 
     def get_mongo_records(self, mongo, filter_dict, workspace, which, actual_to_user,
-            col_dict=None, col_names_are_external=True, flatten_records=True, args=None):
+            col_dict=None, col_names_are_external=True, flatten_records=True, need_alive=False, args=None):
 
         started = time.time()
 
@@ -304,6 +315,7 @@ class ReportBuilder():
         reverse = utils.safe_value(args, "reverse")
         # use MONGO to do all of the work (query, sort, first/last)
         sort_col = utils.safe_value(args, "sort", "name")
+        sort_dir = 1
 
         if sort_col:
             if sort_col == "name":
@@ -364,7 +376,7 @@ class ReportBuilder():
 
         if flatten_records:
             # pull out requested cols, flattening nested values to their dotted names
-            records = self.flatten_records(records, sort_col, args)
+            records = self.flatten_records(records, sort_col, need_alive, args)
 
         if last:
             # we had to reverse the sort done by mongo, so correct it here
@@ -654,11 +666,7 @@ class ReportBuilder():
 
                 # special formatting for time values
                 if col in duration_col_names:
-                    value = float(value)   # in case its a string
-                    value = str(datetime.timedelta(seconds=value))
-                    index = value.find(".")
-                    if index > -1:
-                        value = value[:index]
+                    value = self.format_duration(value, col, record)
                 elif col in time_col_names:
                     if isinstance(value, str):
                         value = arrow.get(value)
@@ -751,12 +759,84 @@ class ReportBuilder():
 
         return text, row_count
 
+    def should_highlight_row(self, highlight_exp, rd):
+        should = False
+
+        if highlight_exp == "$alive":
+            status = utils.safe_value(rd, "status")
+            should = status in ["queued", "running"]
+
+        return should
+
+    def format_duration(self, value, col, record):
+        # value = str(datetime.timedelta(seconds=value))
+        # index = value.find(".")
+        # if index > -1:
+        #     value = value[:index]
+
+        ongoing = False
+        status = utils.safe_value(record, "status")
+        if not value and status in ["queued", "running"]:
+            value = 0
+            start = None
+
+            # need to compute on the fly
+            if col == "queued":
+                start = utils.safe_value(record, "created")
+            elif col == "duration":
+                start = utils.safe_value(record, "started")
+
+            if start:
+                start_value = utils.get_time_from_arrow_str(start)
+                value = time.time() - start_value
+                ongoing = True
+
+        if not value:
+            value = ""
+        else:
+            secs = float(value)   # in case its a string
+            secs_per_day = 60*60*24
+            days = secs/secs_per_day
+            plus = "+" if ongoing else ""
+
+            if days >= 1:
+                value = "{}{:,.1f} days".format(plus, days)
+            else:
+                hrs = days*24
+                if hrs >= 1:
+                    value = "{}{:,.1f} hrs".format(plus, hrs)
+                else:
+                    mins = hrs*60
+                    value = "{}{:,.1f} mins".format(plus, mins)
+
+        return value
+
+    def get_report_color(self, config_section, config_name):
+        color_name = self.config.get(config_section, config_name).upper()
+        color = getattr(pc_utils, color_name) if color_name else pc_utils.NORMAL
+
+        return color
+
     def generate_report(self, col_infos, records, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
         col_space, duration_col_names, time_col_names, significance, max_fixed_length):
 
         # process COLUMN HEADERS
         text = ""
         first_col = True
+        if self.config:
+            color_highlight = self.get_report_color("run-reports", "color-highlight")
+            color_hdr = self.get_report_color("run-reports", "color-hdr")
+            highlight_exp = self.config.get("run-reports", "highlight")
+        else:
+            color_highlight = None
+            highlight_exp = None
+            color_hdr = None
+
+        if color_highlight or color_hdr:
+            pc_utils.enable_ansi_escape_chars_on_windows_10()
+
+        if color_hdr:
+            text += color_hdr
 
         for col_info in col_infos:
             if first_col:
@@ -765,9 +845,11 @@ class ReportBuilder():
                 text += " " * col_space
 
             user_fmt = col_info["user_fmt"] 
-            right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt)
             col_width = col_info["col_width"]
             col_name = col_info["user_name"].upper() if uppercase_hdr_cols else col_info["user_name"]
+            
+            right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt) or \
+                col_info["name"] in duration_col_names
 
             if truncate_with_ellipses and len(col_name) > col_width:
                 col_text = col_name[0:col_width-3] + "..."
@@ -782,6 +864,9 @@ class ReportBuilder():
 
             text += col_text
 
+        if color_hdr:
+            text += pc_utils.NORMAL
+
         header_line = text
         text += "\n\n"
         row_count = 0
@@ -789,6 +874,12 @@ class ReportBuilder():
         # process VALUE ROWS
         for row_num, record in enumerate(records):
             first_col = True
+            highlighted = False
+
+            if highlight_exp and color_highlight:
+                if self.should_highlight_row(highlight_exp, record):
+                    text += color_highlight
+                    highlighted = True
 
             if row_num % 500 == 0:
                 console.diag("build_formatted_table: processing row: {}".format(row_num))
@@ -800,9 +891,12 @@ class ReportBuilder():
                     text += " " * col_space
 
                 user_fmt = col_info["user_fmt"] 
-                right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt or col_info["value_type"] == bool)
-                col_width = col_info["col_width"]
                 col = col_info["name"]
+
+                right_align = right_align_num_cols and (col_info["is_numeric"] or user_fmt or \
+                    col_info["value_type"] == bool) or col in duration_col_names
+
+                col_width = col_info["col_width"]
                 align = ">" if right_align else "<"
 
                 if not col in record:
@@ -816,11 +910,7 @@ class ReportBuilder():
 
                     # special formatting for time values
                     if col in duration_col_names:
-                        value = float(value)   # in case its a string
-                        value = str(datetime.timedelta(seconds=value))
-                        index = value.find(".")
-                        if index > -1:
-                            value = value[:index]
+                        value = self.format_duration(value, col, record)
                     elif col in time_col_names:
                         if isinstance(value, str):
                             value = arrow.get(value)
@@ -865,6 +955,10 @@ class ReportBuilder():
                         value = "" if value is None else str(value)
                         str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
                         text += str_fmt.format(value)
+
+            if highlighted:
+                text += pc_utils.NORMAL
+
             text += "\n"
             row_count += 1
         
