@@ -6,7 +6,7 @@
 import os
 import sys
 import json
-import arrow
+import atexit
 import random
 from collections import OrderedDict
 
@@ -30,7 +30,7 @@ FN_CHECKPOINT_WILD = "checkpoints/*"
 class Run():
 
     def __init__(self, config=None, store=None, xt_logging=True, aml_logging=True, checkpoints_enabled=True,
-        tensorboard_path=None, supress_normal_output=False):
+        tensorboard_path=None, supress_normal_output=False, auto_close=True):
         ''' 
         this initializes an XT Run object so that ML apps can use XT services from within their app, including:
             - hyperparameter logging
@@ -49,6 +49,10 @@ class Run():
         self.metric_names = OrderedDict()
         self.supress_normal_output = supress_normal_output
         self.user_requests = []
+        self.closed = False
+
+        if auto_close:
+            atexit.register(self.close)
 
         # tensorboard writers
         self.train_writer = None
@@ -231,7 +235,11 @@ class Run():
         return child_run
 
     def close(self):
-        
+
+        if self.closed:
+            # avoid calling close twice (once by app, once by auto_close)
+            return
+
         if self.xt_logging and self.direct_run and self.store and self.context:
             context = self.context
             status = "completed"
@@ -249,6 +257,10 @@ class Run():
 
         self.store.mongo.on_run_close()
 
+        if self.store and self.store.mongo:        
+            print("RUN: mongo call stats")
+            self.store.mongo.print_call_stats()
+
         if self.train_writer:
             self.train_writer.close()
             self.test_writer.close()
@@ -267,18 +279,19 @@ class Run():
             # TODO: how to do this partial log for killed/error runs?
             status = "completed"   
             exit_code = 0
-            restarts = None
             hparams_dict = None
             metrics_rollup_dict = None
             end_time = utils.get_time()
             log_records = []
 
             self.store.end_run(self.ws_name, self.run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
-                restarts=restarts, aggregate_dest=None, dest_name=None, is_aml=True)
+                aggregate_dest=None, dest_name=None, is_aml=True)
 
-            self.store.update_run_at_end(self.ws_name, self.run_name, status, exit_code, restarts, end_time, log_records, 
+            self.store.update_run_at_end(self.ws_name, self.run_name, status, exit_code, end_time, log_records, 
                 hparams_dict, metrics_rollup_dict)
 
+        self.closed = True
+
     def get_store(self):
         return self.store
 
