@@ -1,4 +1,4 @@
-#
+    #
 # Copyright (c) Microsoft Corporation.
 # Licensed under the MIT license.
 #
@@ -13,17 +13,19 @@ from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import job_helper
+from xtlib.helpers import file_helper
 from xtlib import store_utils
 
 from xtlib.console import console
 
 run_info_props = {"_id": 1, "app_name": 1, "box_name": 1, "compute": 1, "create_time": 1, "description": 1, "exper_name": 1,
     "from_ip": 1, "from_computer_name": 1, "is_child": 1, "is_parent": 1, "is_outer": 1, "job_id": 1, 
-    "node_index": 1, "path": 1, "repeat": 1, "script": 1, "run_name": 1, "run_num": 1, "run_guid": 1,
+    "node_index": 1, "path": 1, "repeat": 1, "script": 1, "run_index": 1, "run_name": 1, "run_num": 1, "run_guid": 1,
     "search_style": 1, "search_type": 1, "service_type": 1, "sku": 1, "username": 1, "ws_name": 1, 
     "xt_build": 1, "xt_version": 1}
 
-run_stats_props = {"status": 1, "last_time": 1, "exit_code": 1, "restarts": 1, "end_time": 1, "end_id": 1, "tags": 1}
+run_stats_props = {"status": 1, "start_time": 1, "last_time": 1, "exit_code": 1, "restarts": 1, 
+    "end_time": 1, "queue_duration": 1, "run_duration": 1, "end_id": 1, "tags": 1, "metrics_names": 1}
 
 def expand_run_list(store, mongo, workspace, name_list):
     '''
@@ -98,8 +100,11 @@ def expand_job_entry(store, mongo, workspace, run_list, job_term):
     job_list, actual_ws = job_helper.expand_job_list(store, mongo, workspace, [job_term], can_mix=False)
 
     run_filter = {"job_id": {"$in": job_list}}
-    records = mongo.get_info_for_runs(workspace, run_filter, {"_id": 1})
+    records = mongo.get_info_for_runs(workspace, run_filter, {"_id": 1, "run_num": 1})
     if records:
+        # records are not in any order from mongo, so sort by run_num
+        records.sort(key=lambda r: r["run_num"])
+
         names = [run["_id"] for run in records]
         run_list += names
 
@@ -169,11 +174,9 @@ def list_run_tags(store, mongo, ws_name, name_list, tag_list):
         console.print("no matching run found")
 
 def remove_run_stats(dd: dict):
-    run_stat_props = ["status", "last_time", "exit_code", "restarts", "end_time", "end_id", 
-        "metric_names", "start_time", "duration", "run_duration", "queue_duration"]
     run_stats = {}
 
-    for prop in run_stat_props:
+    for prop in run_stats_props.keys():
         if prop in dd:
             value = dd[prop]
             run_stats[prop] = value
@@ -188,13 +191,17 @@ def get_run_property_dicts():
         "from_host": "from_computer_name", "from_ip": "from_ip", 
         "guid": "run_guid", "job": "job_id", "last_time": "last_time",
         "node": "node_index", "outer": "is_outer", "parent": "is_parent", "path": "path", 
-        "pool": "pool", "repeat": "repeat", "restarts": "restarts", "run": "run_name", "run_num": "run_num", 
-        "script": "script", "search": "search_type", "search_style": "search_style", "service_type": "service_type", "sku": "sku",
-        "status": "status", "target": "compute", "username": "username", "vc": "vc", "workspace": "ws_name", "xt_build": "xt_build", 
+        "pool": "pool", "repeat": "repeat", "restarts": "restarts", "run": "run_name", "run_index": "run_index", 
+        "run_num": "run_num", "script": "script", "search": "search_type", "search_style": "search_style", 
+        "service_type": "service_type", "sku": "sku", "status": "status", "target": "compute", 
+        "username": "username", "vc": "vc", "workspace": "ws_name", "xt_build": "xt_build", 
         "xt_version": "xt_version",
+
+        # embedded properties
         "hparams": "hparams", "metrics": "metrics", "tags": "tags",
 
-         "ended": "end_time", "started": "start_time", "duration": "run_duration", "queued": "queue_duration",
+        # run stats
+        "ended": "end_time", "started": "start_time", "duration": "run_duration", "queued": "queue_duration",
 
          # special info (not a simple property)
          "metric_names": "metric_names",
@@ -208,6 +215,8 @@ def get_run_property_dicts():
         "cluster": "the name of the service cluster for the run", 
         "ended": "when the run execution ended",
         "description": "the user specified description associated with the run",
+        "duration": "how long the run has been executing",
+        "ended": "when the run completed",
         "experiment": "the name of the experiment associated with the run",
         "exit_code": "the integer value returned by the run",
         "from_host": "the name of the computer that submitted the run",
@@ -215,15 +224,18 @@ def get_run_property_dicts():
         "guid": "a string that uniquely identifies the run",
         "job": "the job id that the run was part of",
         "last_time": "the time of the most recent operation associated with the run",
-        "run": "the name of the run",
-        "run_num": "the sort-compatible number portion of run",
+        "metric_names": "list of metrics names, ordered by their reporting",
         "node": "the 0-based node index of the run's box",
         "outer": "indicates this run is not a child run",
         "parent": "indicates that the run spawned child runs",
         "path": "the full path of the run's target script or executable file",
         "pool": "the user-defined name describing the backend service or set of boxes on which the run executed",
+        "queued": "how long the run was waiting to start",
         "repeat": "the user-specified repeat-count for the run",
         "restarts": "the number of times the run was preempted and restarted",
+        "run": "the name of the run",
+        "run_index": "the job command index assigned to this run",
+        "run_num": "the sort-compatible number portion of run",
         "script": "the base name of the run's script or executable file",
         "search": "the type of hyperparameter search used to start the run",
         "search_style": "describes how search is being accomplished (one of: static, dynamic, multi, repeat, single)",
@@ -237,14 +249,6 @@ def get_run_property_dicts():
         "workspace": "the name of the workspace containing the run",
         "xt_build": "the build date of xt that was used to launch the run",
         "xt_version": "the version of xt that was used to launch the run",
-
-        "ended": "when the run completed",
-        "started": "when the run started running",
-        "duration": "how long the run has been executing",
-        "queued": "how long the run was waiting to start",
-
-        "metric_names": "list of metrics names, ordered by their reporting",
-
         }
 
     return user_to_actual, std_cols_desc
@@ -376,10 +380,13 @@ def get_filtered_sorted_limit_runs(store, config, show_gathering, col_dict=None,
     # if show_gathering:
     #     console.print("gathering run data...", flush=True)
 
+    highight_exp = config.get("run-reports", "highlight") 
+    need_alive = highight_exp == "$alive"
+
     # get the mongo records for the matching RUNS
     records, using_default_last, last = builder.get_mongo_records(mongo, filter_dict, workspace, which="runs", 
         actual_to_user=actual_to_user, col_dict=col_dict, col_names_are_external=col_names_are_external, 
-        flatten_records=flatten_records, args=args)
+        flatten_records=flatten_records, need_alive=need_alive, args=args)
 
     # remove key info from _id in records
     store_utils.simplify_records_id(records)
@@ -858,7 +865,20 @@ def wrapup_runs_by_node(store, workspace, job_id, node_id, node_cancel_result):
     ws_name = job_info["ws_name"]
 
     # loads the controller's MRC for context of this node
-    text = store.read_job_file(workspace, job_id, constants.FN_MULTI_RUN_CONTEXT)
+    if store.does_job_file_exist(workspace, job_id, constants.FN_MULTI_RUN_CONTEXT):
+        text = store.read_job_file(workspace, job_id, constants.FN_MULTI_RUN_CONTEXT)
+    else:
+        # legacy job (need to download xt_code.zip and extract MRC file)
+        tmp_dir = file_utils.make_tmp_dir("unzip_code")
+        fn_zip_local = "{}/{}".format(tmp_dir, constants.CODE_ZIP_FN)
+        store_path = "before/code/{}".format(constants.CODE_ZIP_FN)
+        store.download_file_from_job(workspace, job_id, store_path, fn_zip_local)
+
+        # now, extract the MRC file
+        file_helper.unzip_file(fn_zip_local, constants.FN_MULTI_RUN_CONTEXT, tmp_dir)
+        fn_mrc = "{}/{}".format(tmp_dir, constants.FN_MULTI_RUN_CONTEXT)
+        text = file_utils.read_text_file(fn_mrc)
+
     mrc_data = json.loads(text)
 
     context_by_nodes = mrc_data["context_by_nodes"]
@@ -869,13 +889,17 @@ def wrapup_runs_by_node(store, workspace, job_id, node_id, node_cancel_result):
     node_index = utils.node_index(node_id)
 
     unwrapped = ["created", "queued", "spawning", "allocating", "running"]
-    filter_dict = {"job_id": job_id, "node_index": node_index, "status":  {"$in": unwrapped}}
-    fields_dict = {"log_records": 0}
+    # filter_dict = {"job_id": job_id, "node_index": node_index, "status":  {"$in": unwrapped}}
+
+    filter_dict = {"job_id": job_id, "node_index": node_index}
+    fields_dict = {"status": 1, "run_index": 1, "run_name": 1 }
 
     runs = store.mongo.get_info_for_runs(ws_name, filter_dict, fields_dict)
 
     for run in runs:
-        wrapup_run_with_context(store, run, context)
+        status = run["status"]
+        if status in unwrapped:
+            wrapup_run_with_context(store, run, context)
 
     return runs
 
@@ -886,7 +910,7 @@ def wrapup_run_with_context(store, run, context_dict):
     node_id = utils.node_id(context.node_index)
 
     # use info from run, when possible (context is shared among all child runs)
-    run_index = run["run_index"]
+    run_index = utils.safe_value(run, "run_index")
     run_name = run["run_name"]
 
     # these we don't have info for
