@@ -187,7 +187,7 @@ def add_controller_env_vars(env_vars, config, box_secret, node_id):
 
     mongo_creds, mongo_name = config.get_mongo_creds()
 
-    ss_info = config.get_service_set_info()
+    ss_info = config.get_store_info()
     storage_tracking = utils.safe_value(ss_info, "track-storage")
 
     if storage_tracking:
