@@ -59,7 +59,7 @@ class MongoDB2():
     '''
     
     def __init__(self, mongo_cs, db_name=None, ws_name=None, run_cache_dir=None, 
-        direct_mode=True, store=None, buffer_metrics=False):
+        direct_mode=True, store=None, mongo_options=None):
 
         '''
         direct_mode: means we update data in mongo directly (from XT or the run) (vs. storge tracking)
@@ -76,13 +76,17 @@ class MongoDB2():
         self.round_trip_count = 0         # stats for last query
         self.name = "mongo_v2"
         self.store = store
-        self.buffer_metrics = buffer_metrics
+        self.last_metrics_write = time.time()
         self.metrics = {}
 
-        # default controls for mongo stats and logging
+        # mongo options for stats and logging
         self.update_job_stats = True
         self.update_run_stats = True
         self.add_log_records = True
+        self.buffer_metrics = 0
+
+        if mongo_options:
+            self.set_mongo_options(mongo_options)
 
         self.mongo_client = MongoClient(mongo_cs)
 
@@ -96,6 +100,11 @@ class MongoDB2():
         self.run_cache_dir = os.path.expanduser(run_cache_dir) if run_cache_dir else None
 
     #---- UTILS ----
+    def set_mongo_options(self, md):
+        self.update_job_stats = md["update-job-stats"]
+        self.update_run_stats = md["update-run-stats"]
+        self.add_log_records = md["add-log-records"]
+        self.buffer_metrics = md["buffer-metrics"]
 
     def get_service_name(self):
         _, rest = self.mongo_conn_str.split("//", 1)
@@ -459,20 +468,35 @@ class MongoDB2():
     def get_collection_records(self, collection, ws_name, filter, fields=None):
         dbx = self.get_db(ws_name)
 
+        # always try mongo first
         records = self.mongo_with_retries("get_collection_records", lambda: dbx[collection].find(filter, fields), 
             return_records=True)
 
-        # experimental code: failover to storage if records not found
-        if not records and collection == "log_records":
+        # support for mongo.add-log-records == False
+        if collection == "log_records":
             id_list = filter["key"]["$in"]
-            records = self.store.get_log_records_for_runs(ws_name, id_list)
+            storage_id_list = self.get_ids_without_logs(id_list, records)
+            records += self.store.get_log_records_for_runs(ws_name, storage_id_list)
 
         return records
+
+    def get_ids_without_logs(self, id_list, records):
+        if records:
+            found_set = set( [r["key"] for r in records] )
+            missing_ids = list(set(id_list) ^ found_set)
+        else:
+            missing_ids = id_list
         
+        return missing_ids
+
+
     def update_collection(self, collection_name, ws_name, dd, dbx=None, is_flat=True, flat_exceptions=[]):
         '''
         add or update a single record in the specified collection
         '''
+        if collection_name == "log_records" and not self.add_log_records:
+            return
+
         if not dbx:
             dbx = self.get_db(ws_name)
 
@@ -767,16 +791,21 @@ class MongoDB2():
             self.update_collection("hparams", ws_name, dd)
 
         elif event == "metrics":
-            metrics_dd = dict(dd)
-            self.add_id(dd, ws_name, run_name)
+            self.metrics.update(dd)
 
-            if self.buffer_metrics:
-                #console.print("buffering metrics: {}".format(dd))
-                self.metrics.update(dd)
-                #console.print("after metrics update: self.metrics=", self.metrics)
+            if not "_id" in self.metrics:
+                self.add_id(self.metrics, ws_name, run_name)
 
+            diff = time.time() - self.last_metrics_write
+
+            if diff/60 > self.buffer_metrics:
+                # time to write metrics to mongo
+                console.print("WRITING metrics: {}".format(dd))
+                self.update_collection("metrics", ws_name, self.metrics)
+                self.last_metrics_write = time.time()
             else:
-                self.update_collection("metrics", ws_name, dd)
+                #console.print("BUFFERING metrics: {}".format(dd))
+                pass
 
         if run_stats:
             self.add_id(run_stats, ws_name, run_name)
@@ -792,48 +821,50 @@ class MongoDB2():
             # at end of run, write out the buffered metrics record
             console.print("writing METRICS at end of run: {}".format(self.metrics))
             ws_name = self.metrics["_id"].split("/")[0]
-            
+
             self.update_collection("metrics", ws_name, self.metrics)
 
     def add_log_record(self, ws_name, run_name, orig_dd):
         '''
         add single log record dict (dd) to LOG_RECORDS collection
         '''
-        dd = dict(orig_dd)    # make local copy that we can update
-        
-        # assign a GUID to each log record's id
-        dd["_id"] = str(uuid.uuid4())
-        dd["ws_name"] = ws_name
+        if self.add_log_records:
+            dd = dict(orig_dd)    # make local copy that we can update
+            
+            # assign a GUID to each log record's id
+            dd["_id"] = str(uuid.uuid4())
+            dd["ws_name"] = ws_name
 
-        key = self.make_key(ws_name, run_name)
-        dd["key"] = key
+            key = self.make_key(ws_name, run_name)
+            dd["key"] = key
 
-        self.update_collection("log_records", ws_name, dd, is_flat=False)
+            self.update_collection("log_records", ws_name, dd, is_flat=False)
 
     def add_log_record_bulk_limited(self, ws_name, run_name, log_records):
         '''
         add multiple log record dicts to LOG_RECORDS collection
         '''
-        dbx = self.get_db(ws_name)
-        key = self.make_key(ws_name, run_name)
+        if self.add_log_records:
+            dbx = self.get_db(ws_name)
+            key = self.make_key(ws_name, run_name)
 
-        max_inserts = 10    # 25 gets request rate errors
-        index = 0
+            max_inserts = 10    # 25 gets request rate errors
+            index = 0
 
-        while index < len(log_records):
-            log_batch = log_records[index : index+max_inserts]
+            while index < len(log_records):
+                log_batch = log_records[index : index+max_inserts]
 
-            for dd in log_batch:
-                # assign a GUID to each log record's id
-                dd["_id"] = str(uuid.uuid4())
-                dd["ws_name"] = ws_name
+                for dd in log_batch:
+                    # assign a GUID to each log record's id
+                    dd["_id"] = str(uuid.uuid4())
+                    dd["ws_name"] = ws_name
 
-                if not "key" in dd:
-                    dd["key"] = key
+                    if not "key" in dd:
+                        dd["key"] = key
 
-            self.mongo_with_retries("add_log_record_bulk_limited", lambda: dbx["log_records"].insert_many(log_batch) )
+                self.mongo_with_retries("add_log_record_bulk_limited", lambda: dbx["log_records"].insert_many(log_batch) )
 
-            index += len(log_batch)
+                index += len(log_batch)
 
     def update_run_stats_from_list(self, ws_name, id_list, dd, clear=False, upsert=True):
         '''
@@ -1032,25 +1063,42 @@ class MongoDB2():
             - set the run "queue_duration" property to NOW - created_time
         '''
         dbx = self.get_db(ws_name)
+        run_restarted = False
 
         if self.update_run_stats:
             now = arrow.now()
             now_str = str(now)
 
-            # get create_time of run
-            cmd = lambda: dbx[ws_name].find({"_id": run_name}, {"create_time": 1})
+            # fetch create_time of run
+            _id = self.make_key(ws_name, run_name)
+            cmd = lambda: dbx["run_info"].find({"_id": _id}, {"create_time": 1, "queue_duration": 1, "restarts": 1})
             records = self.mongo_with_retries("run_start", cmd, return_records=True)
 
             doc = records[0] if records else None
-            if doc and "create_time" in doc:
+            create_time_str = utils.safe_value(doc, "create_time")
+            queue_duration = utils.safe_value(doc, "queue_duration")
+            restarts = utils.safe_value(doc, "restarts")
+
+            # if queue_duration is non-zero, we have started this run previously
+            run_restarted = bool(queue_duration) or restarts > 0
+            if run_restarted:
+                # increment the "restarts" field of run_stats
+                restarts += 1
+                console.print("===> restart #{} detected: {} <======".format(restarts, run_name))
+
+            if create_time_str:
                 create_time_str = doc["create_time"]
                 create_time = arrow.get(create_time_str)
 
                 # compute time in "queue" 
                 queue_duration = utils.time_diff(now, create_time)
 
-                cmd = lambda: dbx[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"start_time": now_str, "queue_duration": queue_duration} })
-                self.mongo_with_retries("run_start", cmd)
+            # update: start_time, queue_duration, restarts
+            cmd = lambda: dbx["run_stats"].find_and_modify( {"_id": _id}, \
+                update={"$set": {"start_time": now_str, "queue_duration": queue_duration, "restarts": restarts} })
+            self.mongo_with_retries("run_start", cmd)
+
+        return run_restarted
 
     def run_exit(self, ws_name, run_name):
         '''
@@ -1063,8 +1111,9 @@ class MongoDB2():
             now = arrow.now()
             now_str = str(now)
 
-            # get start_time of run
-            cmd = lambda: dbx[ws_name].find({"_id": run_name}, {"start_time": 1})
+            # fetch start_time of run
+            _id = self.make_key(ws_name, run_name)
+            cmd = lambda: dbx["run_stats"].find({"_id": _id}, {"start_time": 1})
             records = self.mongo_with_retries("run_exit #1", cmd, return_records=True)
 
             doc = records[0] if records else None
@@ -1075,7 +1124,8 @@ class MongoDB2():
                 # compute run_duration 
                 run_duration = utils.time_diff(now, start_time)
 
-                cmd = lambda: dbx[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"run_duration": run_duration} })
+                cmd = lambda: dbx["run_stats"].find_and_modify( {"_id": _id}, \
+                    update={"$set": {"run_duration": run_duration} })
                 self.mongo_with_retries("run_exit #2", cmd)
 
     def job_node_start(self, ws_name, job_id):
@@ -1120,7 +1170,7 @@ class MongoDB2():
 
         return job_completed
 
-    def update_run_at_end(self, ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics):
+    def update_run_at_end(self, ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics):
         # update run document on Mongo DB
         dbx = self.get_db(ws_name)
 
@@ -1129,7 +1179,6 @@ class MongoDB2():
             updates = {}
             updates["status"] = status
             updates["exit_code"] = exit_code
-            updates["restarts"] = restarts
             updates["end_time"] = end_time
 
             # add the unique end_id (relative to ws_name)
@@ -1839,7 +1888,7 @@ class MongoRunIndex():
     '''
     used by Controller to manage scheduling of child runs.
     '''
-    def __init__(self, mongo, ws_name, job_id, parent_run_name, node_id, new_session=True):
+    def __init__(self, mongo, ws_name, job_id, parent_run_name, node_id):
         self.ws_name = ws_name
         self.mongo = mongo
         self.job_id = job_id
@@ -1847,8 +1896,7 @@ class MongoRunIndex():
         self.node_id = node_id
         self.node_index = utils.node_index(node_id)
         
-        if new_session:
-            self.mongo.reset_child_runs(ws_name, job_id, self.node_index)
+        self.mongo.reset_child_runs(ws_name, job_id, self.node_index)
 
     def get_next_child_run(self):
         entry = self.mongo.get_next_child_run(self.ws_name, self.job_id, self.node_index)     
