@@ -82,7 +82,7 @@ class JobTracker():
                 if self.store.helper.provider.does_blob_exist(self.ws_name, self.fn_info):
                     info_text = store.helper.provider.get_blob_text(self.ws_name, self.fn_info)
                     ji = json.loads(info_text)
-                    self.mongo.update_job(ji)
+                    self.mongo.create_mongo_job(ji)
 
             # just take new portion of new_text
             old_len = len(self.current_text)
@@ -180,7 +180,7 @@ class StorageTrackingDB():
         except BaseException as ex:
             console.print("error reading run log for {}, ex: {}".format(run_name, ex))
             return
-            
+
         job_id = None
 
         # we accumulate info for these over multiple records
