@@ -6,13 +6,16 @@
 import numpy as np
 import os
 import sys
+import math
 import time
 import json
 import uuid
+import tqdm
 import shutil
 import socket 
 import logging
 from multiprocessing import Pool
+from threading import Thread, Lock
 
 from xtlib import utils
 from xtlib import errors
@@ -37,6 +40,7 @@ from .store_objects import StoreBlobObjs
 #from azureml.core import Workspace
 #from azureml.core import Experiment
 
+# globals
 logger = logging.getLogger(__name__)
 
 # public function
@@ -45,16 +49,16 @@ def create_from_props_dict(pd):
 
 # public function
 def store_from_context(context):
-    store = Store(context.store_creds, provider_code_path=context.store_code_path, mongo_conn_str=context.mongo_conn_str)
 
-    # set switches for mongo stats, logging, and scheduling
-    mongo = store.mongo
-
-    if mongo:
-        mongo.update_job_stats = context.update_job_stats
-        mongo.update_run_stats = context.update_run_stats
-        mongo.add_log_records = context.add_log_records
-        mongo.buffer_metrics = context.buffer_metrics
+    # build mongo_options from context
+    mongo_options = {}
+    mongo_options["update-job-stats"] = context.update_job_stats
+    mongo_options["update-run-stats"] = context.update_run_stats
+    mongo_options["add-log-records"] = context.add_log_records
+    mongo_options["buffer-metrics"] = context.buffer_metrics
+    
+    store = Store(context.store_creds, provider_code_path=context.store_code_path, 
+        mongo_conn_str=context.mongo_conn_str, mongo_options=mongo_options)
 
     return store
 
@@ -88,7 +92,7 @@ def is_version1_store(config=None, storage_creds=None, provider_code_path=None):
 
 # public function
 def Store(storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
-        provider_code_path=None, validate_creds=False):
+        provider_code_path=None, validate_creds=False, mongo_options=None):
     '''
     This function peeks at the version number in the storage and chooses StoreV1 or StoreV2 as the store 
     class to return an instance of.
@@ -114,7 +118,7 @@ def Store(storage_creds=None, config=None, max_retries=25, run_cache_dir=None, f
         assert not provider.does_container_exist(constants.INFO_CONTAINER_V1)
 
         store = StoreV2(storage_creds, config, max_retries, run_cache_dir, feedback_enabled,  mongo_conn_str, 
-            provider_code_path, validate_creds)
+            provider_code_path, validate_creds, mongo_options=mongo_options)
 
         assert not provider.does_container_exist(constants.INFO_CONTAINER_V1)
 
@@ -144,7 +148,7 @@ class StoreV2():
     '''
     
     def __init__(self, storage_creds=None, config=None, max_retries=25, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
-        provider_code_path=None, validate_creds=False):
+        provider_code_path=None, validate_creds=False, mongo_options=None):
         '''This is the constructor for the Store class. '''
 
         # # debug issue of using correct storage account on compute node
@@ -157,6 +161,8 @@ class StoreV2():
             mongo_creds, mongo_name = config.get_mongo_creds()
             mongo_conn_str = mongo_creds["mongo-connection-string"]
 
+        self.max_log_workers = utils.safe_value(mongo_options, "max-log-workers")
+
         if config:
             if not storage_creds:
                 storage_creds = config.get_storage_creds()
@@ -177,7 +183,6 @@ class StoreV2():
                 errors.service_error("Azure Storage service credentials not set correctly" + 
                     "; use 'xt config' to correct")
 
-        self.mongo = mongo_v2.MongoDB2(mongo_conn_str, run_cache_dir=run_cache_dir, store=self) if mongo_conn_str else None
         self.mongo_conn_str = mongo_conn_str
         self.run_cache_dir = run_cache_dir
         self.provider_code_path = provider_code_path
@@ -189,7 +194,11 @@ class StoreV2():
         self.cap_stdout = None
         self.cap_stderr = None
 
-        if self.mongo:
+        if mongo_conn_str:
+
+            self.mongo = mongo_v2.MongoDB2(mongo_conn_str, run_cache_dir=run_cache_dir, store=self, 
+                mongo_options=mongo_options) 
+
             self.helper.validate_storage_and_mongo(self.mongo)
 
     def init_store_utils_if_needed(self):
@@ -603,7 +612,7 @@ class StoreV2():
         self.start_run_core(ws_name, child_name, exper_name=exper_name, description=description, username=username,
             box_name=box_name, app_name=app_name, repeat=None, is_parent=False, job_id=job_id, pool=pool, node_index=node_index,
             aggregate_dest=aggregate_dest, path=path, aml_run_id=None, compute=compute, service_type=service_type, search_type=search_type,
-            sku=sku, search_style=search_style, is_child=True)           
+            sku=sku, search_style=search_style, is_child=True, run_index=run_index)           
 
         # finally, add a "child_created" record to the parent
         self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name}, is_aml=is_aml)
@@ -611,7 +620,7 @@ class StoreV2():
     def start_run_core(self, ws_name, run_name, exper_name=None, description=None, username=None,
             box_name=None, app_name=None, repeat=None, is_parent=False, job_id=None, pool=None, node_index=None,
             aggregate_dest="none", path=None, aml_run_id=None, compute=None, service_type=None, search_type=None,
-            sku=None, search_style=None, is_child=False):
+            sku=None, search_style=None, is_child=False, run_index=None):
 
         console.diag("start_run: after create_next_run_directory")
 
@@ -656,10 +665,12 @@ class StoreV2():
             "node_index": node_index, 
             "path": path, 
             "repeat": repeat, 
-            "script": script, 
+            "restarts": 0,
+            "run_index": run_index, 
             "run_name": run_name, 
             "run_num": run_num, 
             "run_guid": run_guid, 
+            "script": script, 
             "search_style": search_style, 
             "search_type": search_type, 
             "service_type": service_type, 
@@ -700,19 +711,18 @@ class StoreV2():
         return items
 
     def end_run(self, ws_name, run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
-        restarts=0, aggregate_dest=None, dest_name=None, is_aml=False):
+         aggregate_dest=None, dest_name=None, is_aml=False):
+
         if not end_time:
             end_time = utils.get_time()
 
         self.log_run_event(ws_name, run_name, "ended", {"status": status, "exit_code": exit_code, \
-            "restarts": restarts, "metrics_rollup": metrics_rollup_dict}, \
-                event_time=end_time, is_aml=is_aml)
+            "metrics_rollup": metrics_rollup_dict}, event_time=end_time, is_aml=is_aml)
 
         if not is_aml:
             # append "end" record to workspace summary log
             end_record = {"ws_name": ws_name, "run_name": run_name, "time": end_time, "event": "end", 
-                "status": status, "exit_code": exit_code, "hparams": hparams_dict, "metrics": metrics_rollup_dict, 
-                "restarts": restarts}
+                "status": status, "exit_code": exit_code, "hparams": hparams_dict, "metrics": metrics_rollup_dict}
 
             text = json.dumps(end_record) + "\n"
 
@@ -746,7 +756,6 @@ class StoreV2():
             log_records = []
             hparams = {}
             metrics = {}
-            restarts = 0
         else:
             # write run to ALLRUNS file
             if aggregate_dest and aggregate_dest != "none":
@@ -765,13 +774,12 @@ class StoreV2():
         
             hparams = self._roll_up_hparams(log_records) 
             metrics = self.rollup_metrics_from_records(log_records, primary_metric, maximize_metric, report_rollup) 
-            restarts = len([rr["event"] for rr in log_records if rr["event"] == "restarted"])
 
-        self.end_run(ws_name, run_name, status, exit_code, hparams, metrics, restarts=restarts, 
+        self.end_run(ws_name, run_name, status, exit_code, hparams, metrics,
             end_time=end_time, aggregate_dest=aggregate_dest, dest_name=dest_name, is_aml=is_aml)
 
         if self.mongo:
-            self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics)
+            self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
 
     def _roll_up_hparams(self, log_records):
         hparams_dict = {}
@@ -927,11 +935,11 @@ class StoreV2():
     def copy_run_files_to_run(self, ws_name, from_run, run_wildcard, to_run, to_path):
         return self.helper.copy_run_files_to_run(ws_name, from_run, run_wildcard, to_run, to_path)
 
-    def update_run_at_end(self, ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics):
+    def update_run_at_end(self, ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics):
         result = None
 
         if self.mongo:
-            result = self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics)
+            result = self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
 
         return result
 
@@ -989,33 +997,96 @@ class StoreV2():
 
         return records
 
+    def set_max_workers(self, value):
+        self.max_log_workers = value
+
     def get_log_records_for_runs(self, ws_name, id_list):
         # use worker processes to make this faster
 
-        # build workitems to interate over
-        work_items = []
-        config = xt_config.get_merged_config()    
-
-        for id in id_list:
-            run_name = id.split("/")[-1]
-            work_items.append( [config, ws_name, run_name] )
-
-        num_workers = 20
-        print("  reading log_records from storage ({} runs, {} worker processes): ".format(len(id_list), num_workers), 
-            end="")
-
-        pool = Pool(processes=num_workers) 
-
-        started = time.time()
-        results = pool.map(worker_read_log_records, work_items)
-        elapsed = time.time() - started
-
-        print(" {:.0f} secs elapsed".format(elapsed))
-
-        # flatten records into single list
         all_run_records = []
-        for records in results:
-            all_run_records += records
+
+        if id_list:
+            started = time.time()
+            pbar = None
+            num_workers = min(len(id_list), self.max_log_workers)
+
+            use = "threads"   # direct, pool, threads, queue
+            # if use == "direct": 
+            #     num_workers = 0
+
+            print("  reading log_records from storage (use={}, {} runs, {} workers)".format(use, len(id_list), num_workers))
+
+            if len(id_list) > 2:
+                pbar = tqdm.tqdm(total=len(id_list), ascii=True, ncols=80, desc="  runs: ")
+
+            # THREADS method
+            threads = []
+            thread_lock = Lock()   
+
+            def thread_worker(ws_name, ids):
+                for id in ids:
+                    run_name = id.split("/")[-1]
+                    results = self.read_log_records(ws_name, run_name)
+                    with thread_lock:
+                        nonlocal all_run_records
+                        all_run_records += results
+                        if pbar:
+                            pbar.update(1)
+
+            # split id_list into num_workers sub-lists
+            per_list = math.ceil(len(id_list)/num_workers)
+            next_index = 0
+
+            for id in id_list:  
+                # build sub-list
+                ids = id_list[next_index:next_index+per_list]
+                next_index += per_list
+
+                thread = Thread(target=thread_worker, args=(ws_name, ids))
+                thread.start()
+                threads.append(thread)
+
+            for thread in threads:
+                thread.join()
+
+            # elif use == "pool" and num_workers: 
+            #     # POOL method
+            #     # build workitems to interate over
+            #     work_items = []
+            #     config = xt_config.get_merged_config()    
+
+            #     for id in id_list:
+            #         run_name = id.split("/")[-1]
+            #         work_items.append( [config, ws_name, run_name] )
+
+            #     pool = Pool(processes=num_workers) 
+
+            #     res = [pool.apply_async(worker_read_log_records, args=(item,), callback=lambda _: pbar.update(1)) \
+            #         for item in work_items]
+
+            #     runs = [p.get() for p in res]
+
+            #     # # sort them according to id_list (since results were collected as they completed)
+            #     # runs.sort(key=lambda r: r["run_name"])
+
+            #     # flatten records into single list
+            #     for run in runs:
+            #         all_run_records += run["records"]
+
+            # elif use == "direct":
+            #     # DIRECT method
+            #     # read one at a time in main process
+            #     for id in id_list:  
+            #         run_name = id.split("/")[-1]
+            #         records = self.read_log_records(ws_name, run_name)
+            #         all_run_records += records
+            #         pbar.update(1)
+
+            if pbar:
+                pbar.close()
+            elapsed = time.time() - started
+
+            console.print("  use: {}, elapsed: {:.2f} secs".format(use, elapsed))
 
         return all_run_records
 
@@ -1281,6 +1352,7 @@ def worker_read_log_records(info):
 
     store = Store(config=config)
     records = store.read_log_records(ws_name, run_name)
-    #print(".", end="", flush=True)
-    return records
+
+    run = {"run_name": run_name, "records": records}
+    return run
 
