@@ -11,12 +11,16 @@ from azure.storage.blob import AppendBlobService, BlockBlobService, PublicAccess
 
 from xtlib import utils
 from xtlib import errors
+from xtlib import console
 from xtlib.storage.store_interface import StoreInterface
 
 logger = logging.getLogger(__name__)
 
+# warn user before we trip over confusing OS error for long paths
+MAX_PATH = 259 if os.name == "nt" else 4096
+
 class AzureBlobStore21(implements(StoreInterface)):
-    def __init__(self, storage_creds, max_retries=10):
+    def __init__(self, storage_creds, max_retries=25):
         self.storage_id = storage_creds["name"]
         self.storage_key = storage_creds["key"]
 
@@ -85,13 +89,15 @@ class AzureBlobStore21(implements(StoreInterface)):
     # ---- CONTAINER interface ----
 
     def does_container_exist(self, container):
-        return self.bs.exists(container)
+        result = self.bs.exists(container)
+        return result
 
     def create_container(self, container):
         if self.bs.exists(container):
             errors.service_error("container already exists: {}".format(container))
 
-        return self.bs.create_container(container)
+        result = self.bs.create_container(container)
+        return result
 
     def list_containers(self):
         containers = self.bs.list_containers()
@@ -100,7 +106,10 @@ class AzureBlobStore21(implements(StoreInterface)):
 
     def delete_container(self, container):
         self._container_check(container, create_if_needed=False)
-        return self.bs.delete_container(container)
+        result = self.bs.delete_container(container)
+
+        assert not self.does_container_exist(container)
+        return result
 
     def get_container_properties(self, container):
         props = self.bs.get_container_properties(container)
@@ -264,6 +273,11 @@ class AzureBlobStore21(implements(StoreInterface)):
             dest_dir = "."
         assert os.path.exists(dest_dir)
         
+        dest_fn = os.path.abspath(dest_fn)
+        path_len = len(dest_fn)
+        if path_len > MAX_PATH:
+            console.print("warning: output file path may be too long for this OS: {}".format(path_len))
+
         # azure storage bug workaround: avoid RETRY errors for 0-length blob 
         blob = self.bs.get_blob_properties(container, blob_path)
         if blob.properties.content_length:
@@ -280,6 +294,7 @@ class AzureBlobStore21(implements(StoreInterface)):
             else:
                 # 0-length text file; just write the file outselves
                 text = ""
+
                 with open(dest_fn, "wt") as outfile:
                     outfile.write(text)
            
