@@ -34,7 +34,7 @@ class StoreBlobObjs():
     suppress the STDERR output from Azures API's when they have an _error to keep our
     console output a bit cleaner (easier to follow what's happening).
     '''
-    def __init__(self, storage_creds, provider_code_path, max_retries=10):
+    def __init__(self, storage_creds, provider_code_path, max_retries=25):
 
         self.provider_name = storage_creds["provider"]
 
@@ -79,19 +79,19 @@ class StoreBlobObjs():
             errors.combo_error("this storage paired with mongo service='{}', but passed connection string for mongo service='{}'".format(  \
                 connected_mongo, paired_mongo))
 
-        if storage_info["storage_format"] != constants.STORAGE_FORMAT:
-            errors.store_error("XT storage format {} expected but found: {}".format(constants.STORAGE_FORMAT, storage_info["storage_format"]))
+        if storage_info["storage_format"] != constants.STORAGE_FORMAT_V2:
+            errors.store_error("XT storage format {} expected but found: {}".format(constants.STORAGE_FORMAT_V2, storage_info["storage_format"]))
 
         if not paired_storage:
-            mongo_info = {"paired_storage": storage_name, "storage_format": constants.STORAGE_FORMAT}
+            mongo_info = {"paired_storage": storage_name, "storage_format": constants.STORAGE_FORMAT_V2}
             mongo.set_mongo_info(mongo_info)
 
         if not paired_mongo:
-            storage_info = {"paired_mongo": connected_mongo, "storage_format": constants.STORAGE_FORMAT}
+            storage_info = {"paired_mongo": connected_mongo, "storage_format": constants.STORAGE_FORMAT_V2}
             self._set_storage_info(storage_info)
 
-        if mongo_info["storage_format"] != constants.STORAGE_FORMAT:
-            errors.store_error("XT mongo format {} expected but found: {}".format(constants.STORAGE_FORMAT, mongo_info["storage_format"]))
+        if mongo_info["storage_format"] != constants.STORAGE_FORMAT_V2:
+            errors.store_error("XT mongo format {} expected but found: {}".format(constants.STORAGE_FORMAT_V2, mongo_info["storage_format"]))
 
         # only check once, (takes .5 secs if already imported)
         # remove this check after all XT users have imported (approx. Dec 2019)
@@ -142,7 +142,7 @@ class StoreBlobObjs():
                 self.provider.create_container(info_container)
 
             # create storage info file
-            info = {"storage_format": constants.STORAGE_FORMAT, "paired_mongo": None}
+            info = {"storage_format": constants.STORAGE_FORMAT_V2, "paired_mongo": None}
             text = json.dumps(info)
             self._create_blob(info_container, constants.STORAGE_INFO_FILE, text)
 
