@@ -343,7 +343,7 @@ class MongoDB():
         for key, value in dd.items():
             updates[dd_name + "." + key] = value
 
-    def update_run_at_end(self, ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics):
+    def update_run_at_end(self, ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics):
         # update run document on Mongo DB
 
         if self.update_run_stats:
@@ -351,7 +351,6 @@ class MongoDB():
             updates = {}
             updates["status"] = status
             updates["exit_code"] = exit_code
-            updates["restarts"] = restarts
             updates["end_time"] = end_time
 
             # add the unique end_id (relative to ws_name)
@@ -827,19 +826,32 @@ class MongoDB():
             now_str = str(now)
 
             # get create_time of run
-            cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"create_time": 1})
+            cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"create_time": 1, "queue_duration": 1, "restarts": 1})
             records = self.mongo_with_retries("run_start", cmd, return_records=True)
 
             doc = records[0] if records else None
-            if doc and "create_time" in doc:
+            create_time_str = utils.safe_value(doc, "create_time")
+            queue_duration = utils.safe_value(doc, "queue_duration")
+            restarts = utils.safe_value(doc, "restarts")
+
+            # if queue_duration is non-zero, we have started this run previously
+            run_restarted = bool(queue_duration) or restarts > 0
+            if run_restarted:
+                # increment the "restarts" field of run_stats
+                restarts += 1
+                console.print("===> restart #{} detected: {} <======".format(restarts, run_name))
+
+            if create_time_str:
                 create_time_str = doc["create_time"]
                 create_time = arrow.get(create_time_str)
 
                 # compute time in "queue" 
                 queue_duration = utils.time_diff(now, create_time)
 
-                cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"start_time": now_str, "queue_duration": queue_duration} })
-                self.mongo_with_retries("run_start", cmd)
+            # update: start_time, queue_duration, restarts
+            cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, \
+                update={"$set": {"start_time": now_str, "queue_duration": queue_duration, "restarts": restarts} })
+            self.mongo_with_retries("run_start", cmd)
 
     def run_exit(self, ws_name, run_name):
         '''
