@@ -656,7 +656,8 @@ class StoreV1():
         return items
 
     def end_run(self, ws_name, run_name, status, exit_code, hparams_dict, metrics_rollup_dict, end_time=None, 
-        restarts=0, aggregate_dest=None, dest_name=None, is_aml=False):
+        aggregate_dest=None, dest_name=None, is_aml=False):
+
         if not end_time:
             end_time = utils.get_time()
 
@@ -666,8 +667,7 @@ class StoreV1():
         if not is_aml:
             # append "end" record to workspace summary log
             end_record = {"ws_name": ws_name, "run_name": run_name, "time": end_time, "event": "end", 
-                "status": status, "exit_code": exit_code, "hparams": hparams_dict, "metrics": metrics_rollup_dict, 
-                "restarts": restarts}
+                "status": status, "exit_code": exit_code, "hparams": hparams_dict, "metrics": metrics_rollup_dict}
 
             text = json.dumps(end_record) + "\n"
 
@@ -701,7 +701,6 @@ class StoreV1():
             log_records = []
             hparams = {}
             metrics = {}
-            restarts = 0
         else:
             # write run to ALLRUNS file
             if aggregate_dest and aggregate_dest != "none":
@@ -720,12 +719,11 @@ class StoreV1():
         
             hparams = self._roll_up_hparams(log_records) 
             metrics = self.rollup_metrics_from_records(log_records, primary_metric, maximize_metric, report_rollup) 
-            restarts = len([rr["event"] for rr in log_records if rr["event"] == "restarted"])
 
-        self.end_run(ws_name, run_name, status, exit_code, hparams, metrics, restarts=restarts, 
+        self.end_run(ws_name, run_name, status, exit_code, hparams, metrics, 
             end_time=end_time, aggregate_dest=aggregate_dest, dest_name=dest_name, is_aml=is_aml)
 
-        self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics)
+        self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
 
     def _roll_up_hparams(self, log_records):
         hparams_dict = {}
@@ -878,8 +876,12 @@ class StoreV1():
     def copy_run_files_to_run(self, ws_name, from_run, run_wildcard, to_run, to_path):
         return self.helper.copy_run_files_to_run(ws_name, from_run, run_wildcard, to_run, to_path)
 
-    def update_run_at_end(self, ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics):
-        return self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics)
+    def set_max_workers(self, value):
+        # nothing to do for v1
+        pass
+
+    def update_run_at_end(self, ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics):
+        return self.mongo.update_run_at_end(ws_name, run_name, status, exit_code, end_time, log_records, hparams, metrics)
 
     #---- LOW LEVEL ----
             
