@@ -10,7 +10,10 @@ import logging
 import subprocess
 
 from xtlib import utils
+from xtlib import errors
 from xtlib import file_utils
+from xtlib import store_utils
+
 from .console import console
 from xtlib.storage import store
 
@@ -26,7 +29,6 @@ class TensorboardReader():
         self.cwd = cwd
 
         console.set_level("normal")
-
         os.chdir(cwd)
 
         self.store = store.create_from_props_dict(store_props_dict)
@@ -35,15 +37,19 @@ class TensorboardReader():
     def create_local_fn_from_template(self, run_record, ws_name, run_name, blob_name):
         return path
 
-    def poll_for_tensorboard_files(self, last_changed, blob_path, start_index, tb_path, run_name):
+    def poll_for_tensorboard_files(self, last_changed, blob_path, start_index, tb_path, job_id, run_name):
         # get all blobs in the run's output dir
+        #console.print("polling blob: container={}, path={}".format(self.ws_name, blob_path))
+
         blobs = self.store.list_blobs(self.ws_name, blob_path, return_names=False)
+        #console.print("blob_names=", blobs)
         download_count = 0
 
-        #console.print("blob_names=", blob_names)
         for blob in blobs:
             # is this a tensorboard file?
             basename = os.path.basename(blob.name)
+            #console.print("polling blob: basename={}, name={}".format(basename, blob.name))
+
             if not basename.startswith("events.out.tfevents"):
                 continue
 
@@ -74,7 +80,7 @@ class TensorboardReader():
                 try:
                     console.print("downloading bn={}, local_fn={}".format(bn, local_fn))
                     file_utils.ensure_dir_exists(file=local_fn)
-                    self.store.download_file_from_run(self.ws_name, run_name, bn, local_fn)
+                    self.store.download_file_from_run(self.ws_name, run_name, bn, local_fn, job_id=job_id)
                     download_count += 1
 
                     if self.print_progress:
@@ -86,6 +92,10 @@ class TensorboardReader():
 
     def run(self):
         console.print("XT Tensorboard Reader process (port={})".format(self.port))
+
+        if not self.run_records:
+            errors.internal_error("No runs specified")
+
         names = [rr["run"] for rr in self.run_records]
         console.print("watching runs: {}".format(", ".join(names)))
         console.print()
@@ -114,11 +124,20 @@ class TensorboardReader():
                 tb_path = rr["tb_path"]
                 
                 #console.print("run_name=", run_name)
-                for root in ["output", "mirrored"]:
-                    blob_path = "runs/" + run_name + "/" + root
-                    start_index = 1 + len("runs/" + run_name)
+                '''
+                - the "output" directory is for live (running) jobs writing files to "output" dir
+                - the "mirrored" directory is for monitoring a directory in the running job
+                - the "after" directory is for jobs that only save their Tensorboard files at end of run (via XT)
+                '''
+                for root in ["output", "mirrored", "after/output"]:
+                    #blob_path = "runs/" + run_name + "/" + root
+                    job_id = rr["job"]
+                    run_path = store_utils.get_run_path(job_id, run_name)
 
-                    count = self.poll_for_tensorboard_files(last_changed, blob_path, start_index, tb_path, run_name)
+                    blob_path = run_path + "/" + root
+                    start_index = 1 + len(run_path)
+
+                    count = self.poll_for_tensorboard_files(last_changed, blob_path, start_index, tb_path, job_id, run_name)
                     download_count += count
 
             poll_count += 1
