@@ -279,10 +279,14 @@ def get_python_value_from_text(part):
 
     return value        
 
-def get_time():
+def get_time(arrow_time=None):
     #return time.time()
     return str(arrow.now()) 
 
+def get_time_from_arrow_str(arrow_str):
+    arrow_time = arrow.get(arrow_str)
+    return arrow_time.datetime.timestamp()
+
 def parse_time(arrow_time_str):
     '''
     parse an datetime string produced by arrow module
