@@ -4,9 +4,8 @@
 #
 # xt_cmds.py: defines the XT commands, arguments, and options
 '''
-XT currently has about 60 commands and 90 arguments/options.  The goal
-of this module is to remove all the explict parsing and usage of the 
-config file, from the actual command definitions and entrypoints.
+This module's main() supports the parsing and execution of an XT command.  It is
+usually called from xt_run, but can be called directly (as is done in the quicktest and grokserver).
 '''
 import os
 import sys
@@ -61,18 +60,12 @@ def get_fn_local_config(args):
 
 def main(cmd=None, new_start_time=None, capture_output=False, basic_mode=False, raise_syntax_exception=True):
     '''
-    This is the XT app, used to manage and scale ML experiments, support various backends (Philly, Azure Batch, Azure ML).
+    Parse and execute the specified cmd.
     '''
     if new_start_time:
         global xt_start_time
         xt_start_time = new_start_time
 
-    import numpy as np
-    seed = 5
-    if seed:
-        np.random.seed(seed)
-        np.random.RandomState(seed)
-
     if cmd:
         cmd = cmd.strip()
 
@@ -159,7 +152,7 @@ def main(cmd=None, new_start_time=None, capture_output=False, basic_mode=False,
         #qfe.remove_hidden_commands()
     
     # hide under-development commands
-    hide_commands  = ["collect_logs", "start_tensorboard", "stop_tensorboard", "zip", "unzip", "wget"]
+    hide_commands  = ["collect_logs", "start_tensorboard", "stop_tensorboard"]
 
     # hide internal cmds (for xt development use only)
     hide_commands.append("generate_help")
