@@ -3,6 +3,11 @@
 # Licensed under the MIT license.
 #
 # xt_run.py: switch between NORMAL and QUICK-START modes
+'''
+This module is the entrypoint for the xt.exe (windows) and xt (linux) command line app.  It's
+job is to choose between the normal mode (calling xt_cmds.main()), or the quickstart mode,
+where the command is passed to the background xt server app.
+'''
 
 # average times to invoke XT and to exit XT
 INVOKE_TIME = .25
@@ -140,7 +145,6 @@ def main(cmd=None, disable_quickstart = False, capture_output=False, mini=False)
                 #console.print(ex)
                 time.sleep(1)
                 retry_count += 1
-
             
     elapsed = time.time() - xt_start_time
     #console.print("(elapsed: {:.2f} secs)".format(elapsed))
